.Dd Tue, Aug 10 2004
.Dt Munger 1
.Sh NAME
.Nm munger
.Nd Lisp specialized for 8-bit text processing
.Sh SYNOPSIS
.Nm munger Oo Ao script Ac Oo Ao args Ac ... Oc Oc
.Sh DESCRIPTION
Munger is a simplified, statically-scoped, interpreted lisp specialized
for writing text processors for 8-bit text.  With Munger the programmer may
write line-by-line filters, if serial access to the text is sufficient, or
the programmer may load text into buffers and have line-oriented random
access to those lines, if that is more convenient.  The standard
descriptors may be redirected onto files and processes.  Buffer lines
may be inserted, removed, overwritten, retrieved in whole or in part, with
or without tabs expanded, searched for matches on regular expressions, read
from and written to files, and piped to and from, or filtered through
processes.
.Pp
Using regular expressions to search through or transform strings is as easy in
Munger as it is in Ex.
.Pp
Basic facilities for doing line-oriented I/O are provided, along with an
optional small set of terminfo-based intrinsics for creating
character-oriented interfaces on cursor-addressable output devices.  The
interpreter also has optional interfaces to the SQLite and FastCGI
libraries, which may be included at compilation time.  For more information
on these two libaries, consult http://www.sqlite.org and
http://www.fastcgi.com respectively.
.Pp
This document begins with an overview of the language, followed by a complete
reference for every intrinsic function and macro, and library function and
macro included in the Munger package.  A list of these precedes the reference
section, immediately after the following overview.
.Pp
Some example programs are included in the source distribution.  They are
installed in (libdir), and are described below.
.Bl -tag -width "transform.munger"
.It cat.munger
is a version of the cat utility, the simplest possible filter.
.It grep.munger
is an egrep-like filter.
.It options.munger
is a module which simplifies the processing of command-line arguments.
.It err.munger
reads GCC output from stdin and embeds any detected error messages
into their corresponding source files, as comments.
.It transform.munger
performs a set of regular expression based substitutions destructively
over a set of files.
.It fmt.munger
is a version of the fmt utility.
.It cgi.munger
is a module to aid in writing CGI scripts.
.It fcgi.munger
is the same module converted to work with FastCGI scripts, used by client.munger.
.It client.fcgi
is a skeletal FastCGI script.
.It tags.munger
Generates a tags file from munger source suitable for use with vi(1)
.It cal.munger
Prints a calender of the current month to stdout.  Demonstrates date 
arithmetic functions.
.It filter.munger
is a simple filter which expands documents with embedded munger code in
them.
.It expander.munger
is a version of the same filter, which runs as a FastCGI script, which,
with a properly configured Apache, allows the creation of dynamic web pages
via the expansion of embedded code within static files.
.It ed.munger
is an ed-like line editor, demonstrating various manipulations of buffer
lines.
.It view.munger
is a file viewer resembling vi when invoked as view.  It demonstrates the
use of the optional character-I/O functions.
.It xmlquery.munger
is a module which may be used to extract data from a document processed
with the XML/mbox/MIME parsers included in the Munger distribution.
.It mbox2munger.munger
will decompose a Berkeley mailbox file into separate files, indexed with an
alist which can be queried for structure and content using the xmlquery.munger
module.
.It mime2munger.munger
will decompose a MIME message into its decoded constituent entities, indexed
with an alist which can be queried for structure and content with the 
xmlquery.munger module.
.It xml2alist.munger
is a minimal XML parser which will convert a standalone XML document into
an alist which may be queried for structure and content with the
xmlqeury.munger module.
.It google.munger
demonstrates using the nc (netcat) utility, which is part of the ports
collection, to "filter" an HTTP request in a buffer into its corresponding
HTTP response.  The program repeatedly prompts the user for a string and
requests a GOOGLE(tm) search via SOAP with the string as the search query,
and lists the matches on stdout.  TO USE THIS PROGRAM ONE MUST HAVE A
GOOGLE WEB SERVICES ACCOUNT.  Accounts are free.  See
http://www.google.com/apis/ for details.
.It tk.munger
demonstrates using the "open_child" intrinsic to establish a full-duplex
connection to a local Tcl/Tk "wish" interpreter to create a GUI front-end
to a Munger program.
.El
.Pp
.Sh IMPLEMENTATION NOTES
.Bl -bullet
.It 
The Munger implementation is designed to be easy to understand and extend,
even for those unfamiliar with functional languages.  It is a naive
implementation, structured like a meta-circular interpreter, written in C,
and will perform best when interpreting programs written in an imperative
programming style.
.It
Munger's strings are 8-bit clean, but some of the intrinsic functions in the
interpreter are not.  It is only ASCII 0 (NUL) which will cause problems.  A
few functions will interpret this character as the end of string sentinel, and
so fail to "see" any characters after an embedded NUL.  This should not be
a problem, as it is unlikely you will want to apply these functions to binary
strings.  The interpreter can, however, read and write strings with embedded
NULs, use them in regular expressions and in buffers.  A full list of 8-bit
safe intrinsics is specified in the section below entitled STRINGS.
.It
There is only one namespace, but namespaces may be easily simulated with
lexical closures and hash tables.
.It
There is a "dynamic_let" to impose dynamically-scoped changes upon globals.
.It
The function position of forms is fully-evaluated and must evaluate to an
intrinsic function or a closure, as in Scheme.  The order of evaluation of the
terms of an application is fixed, from left to right, and may be relied upon in
computations.
.It
Munger does not recognize tail-calls, however you may explicitly request
tail-calls with the "tailcall" intrinsic.  This mechanism allows even
anonymous functions to be tail-recursive, and can be used to turn non-tail
positions into tail positions.  Using function calls, whether they be
tail-calls or not, to perform iteration in Munger will always orders of
magnitude slower than using one of the looping intrinsics.
.It
As in Perl, all values are considered to be boolean "true" values except for
the empty list, the empty string, and zero.  There is no NIL object, nor #t or
#f.
.It
All functions return a value.  There are no undefined or unspecified return
values used in this interpreter.
.It
Improper lists cannot be formed in Munger.  The final "cdr" of all lists is
the empty list.
.It
The empty list is a constant which evaluates to itself.  Every empty
list is identical to every other empty list, which is to say they are "eq"
to each other.
.It
"eq" in Munger behaves similarly to "eql" in Common Lisp.  A true "eq", which
only returns a true value when comparing an object to itself, has very
limited use, and therefore I have not bothered to include one.  There is an
"equal" as well, which does what you think it does.  It is defined in 
(join "/" (libdir) "library.lsp").
.It
"eval" does not contain the lisp reader, therefore applying "eval" to
a string will not parse lisp code in the string, but simply cause the
original string to be returned (strings are constants).
.It
There are no destructive list operations.  Munger encourages (forces,
actually) the programmer to use simple constructive list manipulations.
.It
Munger's looping constructs are modelled on those of C.  If you love Lisp's
loops, you'll find Munger's constrictive.  The most efficient way to iterate
is to use the "for", "iterate", or "loop" intrinsics.
.It
The set of intrinsics is purposefully-limited to a minimal set useful
for text processing.  There are many intrinsics with names similar to
those of other lisp dialects, which do not perform similar tasks.  Be
forewarned.
.It
Symbol syntax is similar to symbol syntax in C, including case-sensitivity.
"let*" therefore, is "letn" in Munger.  If you are used to hyphenating symbol
names, you will have to get used to using the underscore in place of the hyphen
to be happy here.
.It
Simple "eval-twice" macros and gensyms are supported.  A macro defintion
differs from a function definition in that the initial "lambda" symbol is
replaced with the "macro" symbol.
.It
Besides lists, three other aggregate types are provided by the language:
one-dimensional dynamically-resizable arrays called "stacks", associative
arrays called "tables", and statically-sized one-dimesional arrays called
"records".
.It
There are no interactive debugging facilities in the interpreter, nor any
editing capabilities at the interpreter prompt, beyond those provided by
the terminal driver (CTRL-H, CTRL-W, CTRL-U).  To debug scripts the `(print
"I made it here!")(exit 1)' technique is used by the author.
.El
.Pp
.Pp
.Pp
.Ss STARTUP
The interpreter attempts to read three files at startup, in order.  The
first, the system library (library.munger), is read from the Munger data
directory (the "libdir" intrinsic will return the fully-qualified path of
the data directory).  Then, if the user has a custom lisp library
(.munger) in his or her home directory, it is read, and lastly, any file
specified by the first command-line argument is read.
.Pp
The succeeding command-line arguments, if present, are considered to be
arguments to the script referenced by the first command-line argument, and
are not accessed by the interpreter.  All the command line arguments may
be accessed from lisp programs with the "current" "rewind", "next", and
"prev" intrinsics, described in the reference section at the end of this
document.
.Pp
.Pp
.Pp
.Ss COMMENTS
If the parser encounters a semicolon (;) or an octothorpe (#), outside of
a string token, the rest of the line from that character to the next
newline or carriage return is considered to be a comment and discarded.
Recognizing the octothorpe as well as the traditional semicolon
comment-character, allows one to put a "shebang" line at the top of one's
scripts, in order to have the system feed the script to the interpreter,
if the script itself is invoked as a command from the shell.
.Pp
.Dl #!/usr/local/bin/munger
.Pp
.Pp
.Pp
.Ss SYMBOLS
The interpreter is case-sensitive when it comes to recognizing all tokens.
Symbol names must consist of only sequences of alphanumeric characters
and the underscore (_), and must not start with numerical characters.
The symbol consisting of a lone underscore is reserved for use by the
"interact" intrinsic and is a constant which cannot be bound to other objects,
although other symbols may be bound to it, which is to say (setq _ 'foobar)
is illegal and will generate an error, while (setq foobar '_) is legal.
.Pp
.Pp
.Pp
.Ss NUMBERS
The interpreter only supports one numerical type, the fixnum.  Fixnums are
fixed-size integers.  Fixnums are constants and evaluate to themselves.
The binary size of the fixnum is one bit less than the word size of the
machine the interpreter is running on.  On a 32-bit machine, the fixnum is
therefore 31 bits.  Arithmetical operations which overflow this size, will
not be detected.  The "maxidx" intrinsic will return the value of the
largest integer the interpreter can represent.  The lowest supported
integer will be one more than the value returned by "maxidx", negated.
This can be demonstrated by adding 1 to the value returned by "maxidx" to
cause the twos-complement representation to "wrap-around" to the negative
side:  
.Bd -literal -offset left
> (setq max (maxidx))
1073741823
> (+ max 1)
-1073741824
.Ed
.Pp
.Pp
.Pp
.Ss STRINGS
Arbitrary strings of character data can be placed between " marks (ASCII
34).  Such tokens are constants and evaluate to themselves. " marks may be
embedded into strings if they are escaped with a backslash:
.Bd -literal -offset left
> "asd\\""
"asd""
.Ed
.Pp
A backslash ocurring at the end of a string will be interpreted as escaping
the closing " character.  To make it possible for the lisp reader to read a
string ending with a backslash, it is necessary, therefore, to have a means
of escaping backslashes.  Backslashes are therefore escaped with
themselves:
.Bd -literal -offset left
> "asd\\\\"
"asd\\"
.Ed
.Pp
Backslashes are interpreted as escapes only when the occur before another
backslash or a " character.  Backslashes not followed by a " character
or another backslash, are inserted into the string.  The following two
strings are therefore identical:
.Bd -literal -offset left
> "\\a"
"\\a"
> "\\\\a"
"\\a"
.Ed
.Pp
It is only necessary to employ these escapes in strings to be parsed by
the lisp reader, that is to say, in the literal strings in your source
code.  These two characters may be inserted into strings programmatically,
without escaping them:
.Bd -literal -offset left
> (stringify (char 34))
"""
> (stringify (char 92))
"\\"
.Ed
.Pp
The interpreter has a small set of string manipulation intrinsics, some you
would expect in any language, such as the "substring" and "strcmp"
functions, and others like the "split", "join", "chomp" and "chop"
intrinsics, which are inspired by similar functions in Perl.  Regular
expressions can be used to find matches on substrings or to transform strings.
There is no character data type.  Single-character strings are used instead.
All string operations are documented in the LANGUAGE REFERENCE section of this
document.
.Pp
Munger's strings are 8-bit clean, but not all of the intrinsics which use
strings will function correctly when confronted with strings wich have
embedded NULs in them.  The following intrinsics will, however:
.Bd -literal -offset left
getline, getchars, print, cgi_read, cgi_print, char, code, substring, regcomp,
match, matches, substitute, replace, stringify, concat, join, chop, chomp,
insert, retrieve, slice, strcmp, child_read, child_write
.Ed
.Pp
The "split" intrinsic, notably, will work correctly when its first argument is
the empty string and its second argument contains NULs.  The programmer should
avoid presenting binary strings to any other intrinsics than those guaranteed
to work with them.
.Pp
.Pp
.Pp
.Ss STACKS, TABLES, AND RECORDS
Munger supports an associative array type called a table.  Associative
arrays are collections of pairs of lisp atoms and arbitrary lisp objects.
The atom is called the "key" and is used to retrieve the other object,
called the "value" from the table.  Tables are implemented internally as
hash tables, hence the name.  The time it takes to retrieve any value from
a particular table is bounded, therefore tables are a more efficient
storage structure than alists for holding key/value pairs.  For more
information on using tables, see the entries for the intrinsics listed
under the heading entitled, Tables, in the LANGUAGE REFERENCE occurring
later in this document.
.Pp
Munger also supports a unidimensional array type called a stack.  Stacks
may be treated as push-down stacks or indexed as arrays.  Internally,
stacks are implemented as unidimensional arrays, but they are called stacks
to emphasize their one-dimensional nature.  The time it takes to access an
element on a stack by its index position, is a constant, therefore stacks
are a more efficient storage structure than lists for holding sequenced
data.  Any lisp object may be stored in a stack.  Multiple-dimension arrays
may be simulated using stacks of stacks.  A stack may be initialized from a
list or flattened to a list.  For more information on using stacks, see the
entries for the intrinsics listed under the heading entitled, Stacks, in the
LANGUAGE REFERENCE occurring later in this document.
.Pp
In addition to stacks and tables, an aggregate type called a record is
provided.  Records are fixed-size unidimensional arrays.  They are more
space-efficient for representing fixed-size structures.  It is recommended
that the programmer bind symbols to fixnums repesenting each of the index
positions in a record, in order to access the record's fields by name.  See
the entries for the "record", "getfield", and "setfield" intrinsics, later
in this document.
.Pp
Tables, stacks, and records are opaque constant atoms which evaluate to
themselves.
.Pp
.Pp
.Pp
.Pp
.Ss FILES
File I/O may be performed in one of two ways.  The content of files may be
read into text buffers, manipulated, then written out again, if random
access to the files' content is desired, or the standard descriptors may be
redirected onto files with the "redirect" intrinsic, if line-oriented,
serial (filter-like) access is sufficient.  Two convenience macros
"with_input_file" and "with_output_file" simplify the process:
.Bd -literal -offset left
> (with_input_file "README"
>> (for (a 1 4) (print a (char 9) (getline))))
1  Munger
2  ======
3  
4  Munger is a simple, statically-scoped, interpreted lisp that has
.Ed
.Pp
All redirections are undone upon return to toplevel.  Redirections made with
"redirect" may be explicitly undone in programs with the "resume" intrinsic.
Redirections made with the "with_input_file" and "with_output_file" macros are
undone automatically when those macro return.  Redirections exhibit stack-like
behavior, allowing nested redirections:  
.Bd -literal -offset left
> (with_input_file "README"
>> (print "Four lines from README:")
>> (newline)
>> (for (a 1 4) (print a (char 9) (getline)))
>> (newline)

>> (with_input_file "lisp.h"
>>> (print "Four lines from lisp.h:")
>>> (newline)
>>> (for (a 1 4) (print a (char 9) (getline)))
>>> (newline))

>> (print "Another four lines from README:"
>> (for (a 1 4) (print a (char 9) (getline))))

Four lines from README:
1  Munger
2  ======
3  
4  Munger is a simple, statically-scoped, interpreted lisp that has

Four lines from lisp.h:
1  /*
2   * Copyright (c) 2001, 2002, 2003, James Bailie <jimmy@jamesbailie.com>.
3   * All rights reserved.
4   * 

Another four lines from README:
1  line-editor-like access to multiple text buffers, for use on the FreeBSD
2  operating system.
3  
4  Installation instructions are in the file INSTALL.  A manual page is
.Ed
.Pp
Two macros simplify the writing of filters even further:  "foreach_line"
and "foreach_line_callback".  Both accept a monadic function to be applied
successively to each line of input, the second also accepts an additional
function to be called when the input source changes.  Both macros determine
whether to read data from the standard input or from files specfied on the
command line based on the existence of command line arguments after the
first two, which will always be the name of the interpreter and the name of
the currently-executing script, respectively.  Here is what "cat" looks
like in Munger:
.Bd -literal -offset left
(next)
(foreach_line print)
(exit 0)
.Ed
.Pp
The example programs further demonstrate the use of these macros.  They are
fully documented lated in this document.
.Pp
.Pp
.Pp
.Ss COMMUNICATING WITH INFERIOR PROCESSES
The interpreter's standard descriptors can be redirected onto processes with
the "pipe" intrinsic.  Successive invocations of "pipe" inherit any
already-made redirections, allowing pipelines to be created.  The
"with_input_process" and "with_output_process" macros simplify the process:  
.Bd -literal -offset left
> (with_input_process "jot 100"
>> (with_input_process "fmt"
>>> (while (setq l (getline))
>>>> (print l))))
1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25
26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47
48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69
70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91
92 93 94 95 96 97 98 99 100
.Ed
.Pp
Upon return to toplevel, all redirections are undone.  Redirections are
explicitly undone with the "resume" intrinsic.  The first argument is
passed to the shell (always /bin/sh), and so may be any expression that
program understands.  Because of this it is actually more efficient to let
the shell create the pipeline for us:
.Bd -literal -offset left
> (with_input_process "jot 100 | fmt" ...
.Ed
.Pp
The standard fetch(1) utility may be used to redirect standard input onto
a remote file:
.Bd -literal -offset left
> (with_input_process "fetch -q -o - 'http://www.jamesbailie.com/index.html'"
>> (print (getline)))
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
.Ed
.Pp
Independently of whatever source the standard descriptors are connected to,
the programmer may launch a program and communicate with it over a
separate full-duplex connection.
.Bd -literal -offset left
> (child_open "munger")
1
> (child_write "(setq foobar 43)")
1
> (chomp (child_read))
"43"
.Ed
.Pp
.Pp
.Pp
.Ss TEXT BUFFERS
Munger provides line-oriented buffers for storing large amounts of text.
A buffer must be opened before it can be used, with the "open" intrinsic, and
may be closed with the "close" intrinsic.  A whole number is returned by
"open" upon success, called the buffer number of the opened buffer.  Only one
buffer, out of those currently open, can be active at any time, and is called
the current buffer.  Each call to "open" creates a new buffer and makes the
new buffer the current buffer.  The buffer number of an open buffer may be
passed as the argument to the "switch" intrinsic to make that buffer the
current buffer.
.Pp
A number of intrinsics are provided which act upon the current buffer, to
insert, retrieve, and delete lines, to report the number of words and lines in
the buffer, and to write and read buffer lines, to and from, files and programs.
Lines may be retrieved whole, or in slices with tabs expanded.  A range of
buffer lines may be processed through an external program with the "filter"
intrinsic, while the "find" intrinsic may be used to search through the current
buffer to find non-overlapping matches on regular expressions.  A range of lines
may be copied from one buffer to another with the "transfer" intrinsic.  There
is also intrinsics to set and find bookmarks in buffers.  The full list of
buffer intrinsics can be found later in this document at the top of the
LANGUAGE REFERENCE section, under the subheading, Buffer Operations.  Each is
fully described in its own particular entry.
.Bd -literal -offset left
; Loading a buffer from a file:

> (open)
0
> (read 0 "README")
38 ; Number of lines read.
> (for (a 1 5) (print (retrieve a)))
Munger
======

Munger is a simple, statically-scoped, interpreted lisp that has
line-editor-like access to multiple text buffers, for use on the FreeBSD

; Loading a buffer from a process:

> (empty)
1
> (input 0 "ls")
20
> (for (a 1 (lastline)) (print (retrieve a)))
LICENSE
Makefile
README
cat.munger
client.munger
err.munger
fcgi.munger
fmt.munger
grep.munger
intrinsics.c
library.munger
lisp.c
lisp.h
options.munger
munger.man
tmltags.munger
transform.munger

; Filtering buffer content through a process:

> (filter 1 (lastline) "fmt")
3 ; Number of lines received back from filter.

> (for (a 1 (lastline)) (print (retrieve a)))
LICENSE Makefile README cat.munger client.munger err.munger fcgi.munger
fmt.munger grep.munger intrinsics.c library.munger lisp.c lisp.h options.munger
munger.man tmltags.munger transform.munger

; Loading a buffer from a remote file using fetch(1):

> (empty)
1
> (input 0 "fetch -q -o 'http://www.jamesbailie.com/index.html'")
144 ; Number of lines read.

; Finding the location of a match on a regular expression:

> (setq rx (regcomp "<body[^>]*>"))
<REGEX#1>
> (find 1 1 0 rx 0)
(21 3 87)
> (slice 21 3 87 1 0)
"<body id="body" onLoad="initialize_fish();initialize_zoom()" onResize="update_width()">"

; Filtering a buffer through an HTTP server using nc(1):

> (empty)
1
> (insert 0 "GET /robots.txt HTTP/1.1" 1)
1
> (insert 1 "Host: www.jamesbailie.com" 1)
1
> (insert 2 "Connection: close" 1)
1
> (insert 3 "" 1)

; Terminate the lines appropriately and filter:

> (for (n 1 (lastline)) (insert n (concat (retrieve n) (char 13) (char 10)) 0))
1
> (filter 1 (lastline) "nc www.jamesbailie.com 80")
12

; Print results:

> (for (n 1 (lastline)) (print (retrieve n)))
HTTP/1.1 200 OK
Date: Sat, 21 Aug 2004 17:04:48 GMT
Server: Apache/1.3.28 (Unix) mod_fastcgi/2.4.0
Last-Modified: Sun, 08 Feb 2004 03:22:17 GMT
ETag: "ee26-22-4025ab69"
Accept-Ranges: bytes
Content-Length: 34
Connection: close
Content-Type: text/plain

User-agent: *
Disallow: /cgi-bin/
.Ed
.Pp
.Pp
.Pp
.Ss REGULAR EXPRESSIONS 
Munger provides five intrinsic functions for working with extended regular
expressions.  All regular expressions must be compiled with the "regcomp"
intrinsic before they may be used by the other regular-expression wielding
intrinsics.  The "match" intrinsic returns a list of two character indices
describing a match.  The "matches" intrinsic returns a list of the text
matched by the regular expression and up to 20 parenthesized
subexpressions.  The "substitute" intrinsic is modelled after the
substitute command from the vi/ex editor, and may be used to perform
regular-expression-based substitution operations upon strings.  the
"replace" library function allows snippets of lisp to be used for the
replacement expression, allowing replacement text to be dynamically
generated for each match in a string.  Lastly, the "find" intrinsic may be
used to find the location of matches against a particular regular
expression in the text in the current buffer.
.Bd -literal -offset left
> (set 'rx (regcomp "munger"))
<REGEXP#1>

> (set 's "/usr/local/share/munger/library.munger")

> (match rx s)
(17 23)

; Text before match:

> (substring s 0 17)
"/usr/local/share/"

; Text of match:

> (substring s 17 (- 23 17))
"munger"

; Text after match:

> (substring s 23 0)
"/library.munger"

> (set 'rx (regcomp "^/?([^/]+/)*([^/]+)"))
<REGEXP#2>

; Full match and matched subexpressions:

> (matches rx s)
("/usr/local/share/munger/library.munger" "munger/" "library.munger"
"" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "")

; Ex-like substitution:

> (substitute rx "\\U\\2" "/usr/local/bin/munger" 1)
"MUNGER"

; Dynamically-generated replacement strings:

; This function uses "replace" to decode x-www-form-urlencoded strings:

(let ((r0 (regcomp "%([0-9A-Fa-f])([0-9A-Fa-f])"))
      (r1 (regcomp "\\+"))
      (hex2decimal 
         (let ((c 0))
            (lambda (x)
               (setq c (code (upcase x 0)))
               (if (and (>= c 65) (<= c 70))
                  (- c 55)
                  (digitize x))))))

   (defun decode (str)  
      (replace r0 
         (char (+ (* 16 (hex2decimal m1)) (hex2decimal m2)))
         (substitute r1 " " str 0))))

> (decode "foobar+tooley%7Efunbag%24%25%26")
"foobar tooley~funbag$%&"
.Ed
.Pp
.Pp
.Pp
.Ss FUNCTIONS
The elements of forms performing function or macro application are
evaluated from left to right, with the function position being evaluated in
the same environment as the succeeding positions.  The function position
must be occupied by an expression that evaluates to a an intrinsic function
object or a closure.  Lambda and macro-expressions evaluate to closure
objects and are closed over the local bindings visible at that time:
.Bd -literal -offset left
> (let ((a 10))
>> (defun booger () a))
<CLOSURE#32>
> (booger)
10
.Ed
.Pp
With static scoping, the local bindings visible to a function are limited
by the block structure in place at the time the function is created, but
variable extent is unlimited, so the function "booger"'s reference to the
enclosing block's variable "a" is still valid after the enclosing block has
returned.  The binding is said to be "closed" within "booger" and is no
longer visible anywhere else.
.Pp
In the example above, the invocation of "defun" creates a toplevel binding,
and not a new locally-visible binding, as an invocation of "define" would
do in this position in Scheme.  Locally-visible bindings can only be
created by function application ("let" is syntactic sugar for the
application of an anonymous function), or by the "extend" intrinsic.  If
the variable "booger" were a local variable in an enclosing scope, its
binding would have been modified, creating a locally-visible function, but
in this case, since no local binding exists, a new toplevel binding is
created.
.Pp
When a lambda-expression is evaluated it is closed over its activation
record and free lexical variables to become a closure.  This means the
symbols in the function's argument list and any symbols visible from other
functions in which it is embedded remain visible and retain their bindings
for as long as the closure exists.  Closures can be used to create shared
lexical environments for functions which need to communicate with each
other, or they can be used to create static bindings which persist between
invocations of a function, much like local variables in C declared with the
"static" storage class.  Because closures perform encapsulation they can
be used to simulate objects in the message-passing style of object
programming.  Closures can also be used to capture state for use in the
continuation-passing style of programming.  A full discussion of these
topics is beyond the scope of this manual page.
.Pp
Functions may be defined to accept variable-length argument lists.  To
specify that a function accepts a varying number of arguments, a final
parameter to reference the optional arguments is enclosed in parentheses in
the parameter list of the function definition.  For example, 
.Bd -literal -offset left
(lambda ((a)) (print a))
.Ed
.Pp
defines a function which accepts zero or more arguments.  Upon invocation,
all arguments passed to the function will be collected into a list and
bound to "a" inside the function body.  If no arguments are passed to the
function, "a" will be bound to the empty list.
.Pp
The function below accepts two or more arguments, with any arguments after
the second argument collected into a list bound to "c" in the function
body:
.Bd -literal -offset left
(lambda (a b (c)) (print a b c))
.Ed
.Pp
The "labels" intrinsic facilitates the creation of local function bindings,
where each binding is visible inside each function definition.  Recursive
and mutually-recursive temporary functions therefore may be created with
"labels".
.Bd -literal -offset left
>(labels ((even (lambda (n) (or (eq n 0) (tailcall odd (- n 1)))))
>>>       (odd  (lambda (n) (and (not (eq n 0)) (tailcall even (- n 1))))))
>> (print (even 11))
>> (newline)
>> (print (even 12))
>> (newline))
0
1
1 ; this is the return value of the last (newline).
.Ed
.Pp
There are "let", "letn", and "letf" macros to facilitate the creation of
new lexical bindings.  "letn" corresponds to let* in other lisp dialects,
while "letf" corresponds to a named let in Scheme or flet in lisp.  It may
not be necessary to use these macros to create new lexical bindings,
depending upon the needs of a particular situation.
.Pp
If the interpreter is a toplevel, then one of these macros must be used, or
a closure applied, to create a lexical environment.  Inside a lexical
enviroment, however, new bindings may be dynamically created with the
"extend" intrinsic.  These bindings have unlimited extent just as all
lexical bindings do, and so may be safely closed-over by closures.
.Bd -literal -offset left
> (defun (a)
>> (extend 'b (* a a))
>> (lambda () b))
.Ed
.Pp
If the programmer wishes to limit the visibility of the new binding, the
invocation of "extend" and the expressions which use the new binding may be
wrapped in a "dynamic_extent" intrinsic:  
.Bd -literal -offset left
> (defun (a)
>> (extend 'f (lambda () b)) 

>> (dynamic_extent
>>> (extend 'b (* a a))
>>> (print (f))) ; Works here inside "dynamic_extent"
                 ; even though f was closed before "extend" was invoked.
                 ; b suddenly "pops-up" into the lexical environment.
>> (f)) ; ERROR: b is no longer extant here.
.Ed
.Pp
If the programmer wishes to both limit the visibility of, and close over,
the new bindings, then one of the "let", "letn", "letf" or "labels" macros
must be employed.  See the entries for the "extend" and "dynamic_extent"
intrinsics, as well as the entries for these macros, later in this
document, for full details.
.Pp
Tail recursion is not recognized by the interpreter, but must be requested
explicitly by the "tailcall" intrinsic.  This mechanism allows even
anonymous functions to perform tail recursion.  See the entry for
"tailcall" later in this document for more details.
.Bd -literal -offset left
> (let ((n 10)
>>>     (a 1))
>> (if (< n 2) 
>>> a
>>> (tailcall 0 (- n 1) (* a n))))
3628800
.Ed
.Pp
.Pp
.Pp
.Ss MACROS
Macros allow us to define syntactic transformations.  With macros we can
create new special forms for the interpreter in lisp.  A macro is simply a
function which receives its arguments unevaluated, and instead has its
return expression evaluated.  In other words, the purpose of a macro is to
compose a new expression out of its arguments, which the interpreter then
evaluates.  Despite this simple definition, the use of macros can become
confusingly complex.  With macros, we write lisp which writes the lisp to
be ultimately evaluated.
.Pp
A simple example is the "quit" macro defined in library.munger:
.Bd -literal -offset left
(set 'quit (macro () '(exit 0)))
.Ed
.Pp
We can see how a macro will expand with the test intrinsic:
.Bd -literal -offset left
> (test (quit))
(exit 0)
.Ed
.Pp
Macros typically consist of a template expression into which we plug
sub-expressions to produce the expression which is finally returned by the
macro.  The "qquote" (quasiquote) macro aids in the defining of other
macros, by allowing us to use the template paradigm directly.  This greatly
simplifies complex macro definitions.  The "qquote" macro is similar to the
"quote" intrinsic in that it prevents evaluation of its argument, but the
quoting can be selectively turned off for sub-expressions by prefacing them
with a (",") comma character.  Here is the definition of the
"with_input_file" macro from library.munger:
.Bd -literal -offset left
(set 'with_input_file
   (macro (file (code))
      (qquote
         (when (> (redirect 0 ,file) 0)
            (protect ,(cons 'progn code)
               (resume 0))))))
.Ed
.Pp
The sub-expressions prefaced by commas are evaluated and the result of the
evaluations are inserted into the template expression.
.Pp
We can see how the macro will expand with the "test" intrinsic:
.Bd -literal -offset left
> (test (with_input_file "library.munger" (getline)))
(when (> (redirect 0 "library.munger") 0)
   (protect (progn (getline))
      (resume 0)))
.Ed
.Pp
There is a "defmac" macro in library.munger which allows macros to be created
with the following syntax:
.Bd -literal -offset left
(defmac with_input_file (file (code))
   (qquote
      (when (> (redirect 0 ,file) 0)
         (protect ,(cons 'progn code)
            (resume 0)))))
.Ed
.Pp
.Pp
.Pp
.Ss PROGRAMMING STYLE
Programs written in an imperative style will always out-perform programs
written in a functional style because Munger is a naive interpreter.  A
Scheme programmer might be tempted to write a factorial function in Munger
like this:  
.Bd -literal -offset left
(defun fact (n)
   (labels ((f (lambda (n a)
                  (if (< n 2)
                     a
                     (f (- n 1) (* n a))))))
      (f n 1)))
.Ed
.Pp
This function will work as expected in Munger, but there are some
improvements we can make to it increase its efficiency.  First of all, the
recursive invocation of "f" will not be recognized as a tail-call by the
Munger interpreter, so we will use use the "tailcall" intrinsic to make
this call.  Secondly, "tailcall" does not need to "see" a binding in order
to make a recursive call.  It can restart the currently-running function
internally.  This means we can use a "let" to make the binding to "f":  
.Bd -literal -offset left
(defun fact (n)
   (let ((f (lambda (n a)
               (if (< n 2)
                  a
                  (tailcall 0 (- n 1) (* n a))))))
      (f n 1)))
.Ed
.Pp
We can simplify this further.  We don't actually need to use "let" to
create new local bindings, unless the interpreter is at toplevel, where no
local lexical environment exists.  The "extend" intrinsic will extend the
current local environment with a new binding more efficiently, because
"let" is a macro internal to the interpreter, which must be expanded and
re-evaluated, resulting in another function application.  If we wish to
limit the extent of the bindings introduced by "extend" we can wrap
expressions with the "dynamic_extent" intrinsic.  Replacing the "let" with
"extend" gives us:  
.Bd -literal -offset left
(defun fact (n)
   (extend 'f 
      (lambda (n a)
         (if (< n 2)
            a
            (tailcall 0 (- n 1) (* n a)))))
   (f n 1))
.Ed
.Pp
We don't use it here, but the binding to "f" is actually visible inside the
function body.  Although the function is closed over the environment before
the environment is extended, the binding will nonetheless be visible when
the closure is applied, because closures do not simply close over visible
bindings, but rather the environments which contain them.  At the time the
closure is applied, the environment will have been dynamically extended to
make "f" visible.
.Pp
Using function calls to iterate will always result in slow programs in
Munger.  The only reason we are introducing the helper function is to
create a new binding to "a" to act as an accumulator, but we can use
"extend" to do that directly, and replace the closure with a loop:  
.Bd -literal -offset left
(defun fact (n)
   (extend 'a 1)
   (while (> n 1)
      (setq a (* n a))
      (dec n))
   a)
.Ed
.Pp
This function is very different than the factorial we started with and will
run much more efficiently in the Munger interpreter.  But there is one last
change we can make to further improve efficiency.  The most efficient way
to iterate a determinate number of times in Munger is to use the "iterate"
intrinsic:  
.Bd -literal -offset left
(defun fact (n)
   (extend 'a 1)
   (when (> n 1)
      (iterate n (setq a (* a n)) (dec n)))
   a)
.Ed
.Pp
This is the fastest form of this function.
.Pp
.Pp
.Pp
.Sh RETURN VALUES
.Pp
.Nm Munger
returns 1 if the interpreter stopped due to an error, 0 otherwise.
.Pp
.Sh AUTHOR
.An James Bailie Aq jimmy@jamesbailie.com
.br
http://www.jamesbailie.com
.Pp
.Sh LANGUAGE REFERENCE
.Pp
Do not assume you know what an intrinsic does because it has a similar name to
an intrinsic of another Lisp dialect, or you may have some nasty surprises
working in Munger.
.Pp
The following functions are either built-in to the interpreter, or are library
functions or macros defined in library.munger.  To find the entry for a
particular item, search this document for the name of the item followed by a
colon.
.Pp
List Operations:
.Bd -literal -offset left
cons,          car,          cdr,           list,     length,
caar,          cdar,         cadr,          cddr,     cdddr,
cddddr,        caddr,        cadddr,        caddddr,  append,
alist_lookup,  alist_remove, alist_replace, reverse,  sort,
sortlist,      sortcar,      mapcar,        foreach,  remove,
nthcdr,        nth,          member,        map
.Ed
.Pp
.Pp
Predicates and Conditionals:
.Bd -literal -offset left
eq,        atom,        if,        when,     unless,
and,       or,          not,       null,     bound,
pair,      equal        <,         <=,       >,
>=,        stringp,     numberp,   symbolp,  regexpp,
tablep,    stackp,      intrinsicp,          closurep,
macrop,    dbp,         recordp
.Ed
.Pp
.Pp
Assignment:
.Bd -literal -offset left
set,       setq,        inc,       dec
.Ed
.Pp
.Pp
Evaluation and Control Flow:
.Bd -literal -offset left
progn,     throw,       while,     until,    do,
catch,     continue,    main,      eval,     quote,
load,      gensym,      version,   test,     let,     
letn,      labels,      cond,      apply,    exit,
quit,      interact,    fatal,     nofatal,  extract,
qquote     protect,     letf,      tailcall, prog1
exec,      printer,     noprinter  die,      dynamic_let,
extend,    gc,          for,       iterate   loop,
dynamic_extent          gc_freq
.Ed
.Pp
.Pp
Math:
.Bd -literal -offset left
+,         -,           *,         /,        %,
abs,       random
.Ed
.Pp
.Pp
Type Conversions:
.Bd -literal -offset left
stringify, digitize,    intern,    char,     code
.Ed
.Pp
.Pp
Buffer Operations:
.Bd -literal -offset left
open,      close,       insert,    delete,    retrieve,
lastline,  filter,      write,     read,      empty,
slice,     find,        input,     output,    words,
maxidx,    buffer,      buffers,   switch,    transfer,
setmark,   getmark,     with_buffer
.Ed
.Pp
.Pp
Regular Expressions:
.Bd -literal -offset left
regcomp,   match,       matches,   substitute
.Ed
.Pp
.Pp
String Operations:
.Bd -literal -offset left
split,     join,        expand,    substring, concat
chop,      chomp,       upcase,    downcase,  length,
reverse,   strcmp,      split_rx,  tokenize,  replace,
rootname,  suffix
.Ed
.Pp
.Pp
Filesystem Operations:
.Bd -literal -offset left
chdir,     libdir,      directory, unlink,    rmdir,
pwd,       exists,      stat,      rename,    seek
mkdir,     complete,    realpath,  access,    truncate,
redirect,  resume,      chown,     chmod,     basename,
readlock,  writelock,   unlock     dirname,   symlink
.Ed
.Pp
.Pp
Command-Line Arguments:
.Bd -literal -offset left
current,   next,        prev,      rewind
.Ed
.Pp
.Pp
Line-Oriented I/O:
.Bd -literal -offset left
print,                  warn,                 getline,
with_input_file,        with_output_file,     with_output_file_appending,
with_input_process,     with_output_process,  foreach_line,           
foreach_line_callback,  pipe                  newline,   
redirect,               resume,               getchars
.Ed
.Pp
.Pp
System Access:
.Bd -literal -offset left
system,    getenv,      setenv,    block,     unblock,
suspend,   lines,       cols,      date,      time,
beep,      checkpass,   crypt,     setuid,    getuid,
geteuid,   seteuid,     getgid,    hostname,  gecos,
timediff,  timethen,    date2days, days2date, date2time,
localtime, week,        weekday,   month
.Ed
.Pp
.Pp
Tables:
.Bd -literal -offset left
table,     hash,        unhash,    lookup,    keys,
values
.Ed
.Pp
.Pp
Stacks:
.Bd -literal -offset left
stack,    push,         pop,       index,    store,
used,     topidx,       assign,    flatten,  shift,    
clear,
unshift
.Ed
.Pp
Records:
.Bd -literal -offset left
record,  getfield,     setfield,
.Ed
.Pp
Communication with an inferior process:
.Bd -literal -offset left
child_open,              child_write,          child_read,
child_close,             child_running,        child_ready,
child_wait,              child_eof
.Ed
.Pp
SQLite Interface:
.Bd -literal -offset left
sql_open,  sql_close,  sql_exec
.Ed
.Pp
FastCGI Interface:
.Bd -literal -offset left
cgi_accept,             cgi_finish,          cgi_xstatus,
cgi_read,               cgi_print,           cgi_newline,
cgi_getenv,             cgi_warn,            cgi_sendfile
.Ed
.Pp
Character-Oriented I/O
.Bd -literal -offset left
display,           clearscreen,    pause,
clearline,         getchar,        goto,
scrolldn,          scrollup,       hide,
show,              pushback,       insertln,
boldface,          normal
.Ed
.Pp
A description of each follows:
.Pp
.Pp
.Pp
.Pp
.Ss cons: (cons expr1 expr2)
Intrinsic "cons" adds an element to the beginning of a list.  Expr2 must
evaluate to a list.
.Bd -literal -offset left
> (cons 'a (b c)
(a b c)
.Ed
.Pp
.Pp
.Pp
.Ss car: (car expr1)
Intrinsic "car" returns the first element of a list. An error is generated
if expr1 does not evaluate to a list.
.Bd -literal -offset left
> (car '(a b c))
a
.Ed
.Pp
.Pp
.Pp
.Ss cdr: (cdr expr1)
Intrinsic "cdr" returns the sublist of a list, beginning from the second
element of the list. An error is generated if expr1 does not evaluate to a
list.
.Bd -literal -offset left
> (cdr '(a b c))
(b c)
.Ed
.Pp
.Pp
.Pp
.Ss bound: (bound expr1)
The "bound " intrinsic accepts one argument which must evaluate to a symbol.
The function returns 1 if the symbol is currently bound, 0 otherwise.
.Pp
.Pp
.Pp
.Ss caar, cadr, cdar, caddr, cadddr, caddddr, cddr, cdddr, cddddr : (form expr)
These four library functions take the same argument as the car and cdr
intrinsics and are built out of nested groupings of those intrinsics.
.Bd -literal -offset left
> (caar '((a) b c)) ; is equivalent to: (car (car '((a) b c)))
a
.Pp
> (cadr '(a b c)) ; is equivalent to: (car (cdr '(a b c)))
b
.Pp
> (cdar '((a) b c)) ; is equivalent to: (cdr (car '((a) b c)))
()
.Pp
> (cddr '(a b c)) ; is equivalent to: (cdr (cdr '(a b c)))
(c)
.Ed
.Pp
   etc.
.Pp
.Pp
.Pp
.Ss eq: (eq expr1 expr2)
Intrisic "eq" returns 1 if expr1 and expr2 evaluate to the same atom
token, to atoms representing equivalent numbers, or to the exact same
list, otherwise it returns 0.
.Bd -literal -offset left
> (eq 'a 'a)
1
.Pp
> (eq '(a b c) '(a b c))
0
.Pp
> (set 'l '(a b c))
(a b c)
> (eq l l)
1
.Pp
> (eq 0001 1)
1
.Ed
.Pp
.Pp
.Pp
.Ss equal: (equal expr1 expr2)
Library function "equal" returns 1 in all the situations where "eq"
returns 1, and additionally will return 1 if both of its arguments
evaluate to lists having the same structure and content.  While "eq" will
fail if its arguments evaluate to different lists with the same structure
and content, equal will return 1 for these arguments..
.Bd -literal -offset left
> (equal '(a b c) '(a b c))
1
.Pp
> (set 'l '(a b c))
(a b c)
> (equal l l)
1
.Pp
> (equal '(00 01 02) '(0 1 2))
1
.Ed
.Pp
.Pp
.Pp
.Ss atom: (atom expr)
Intrinsic "atom" returns 1 if its argument evaluates to an atom, and returns
0 otherwise.
.Bd -literal -offset left
> (atom 'a)
1
.Pp
> (atom '(a b c))
0
.Ed
.Pp
.Pp
.Pp
.Ss set: (set expr1 expr2)
Intrinsic "set" accepts two arguments.  The result of evaluating the second
argument is bound to the result of evaluating the first argument.  The first
argument must evaluate to a symbol.  If a local variable with the syntax of
the symbol exists, its binding will be modified, otherwise "set" will create
or modify a toplevel binding.  Locals can only be created by function
application, or with the "extend" intrinsic.
.Bd -literal -offset left
> (set 's '(a b c))
(a b c)
> s
(a b c)
.Pp
> (set (car '(a b c)) ((lambda (x) (* x x)) 4))
16
> a
16
.Ed
.Pp
.Pp
.Pp
.Ss setq: (setq symbol expr)
The "setq" intrinsic works similarly to the "set" intrinsic, except the
first argument to the function is not evaluated and must be a symbol.
.Bd -literal -offset left
(setq a b)
.Pp
is equivalent to:
.Pp
(set 'a b)
.Ed
.Pp
.Pp
.Pp
.Ss eval: (eval expr)
Intrinsic "eval" returns the result of evaluating its lone argument twice,
which is to say, the argument is evaluated as usual, and the result of this
evaluation is evaluated again.  Note "eval" does not contain the lisp reader.
Calling "eval" with a string argument will only cause the original string to
be returned, since strings are constants in Munger.
.Bd -literal -offset left
> (set 'a (quote (set 'b 'booger)))
(set (quote b) (quote (booger)))
> b
evaluate: b has no value.
> (eval a)
booger
> b
booger
.Ed
.Pp
.Pp
.Pp
.Ss quote: (quote expr) or 'expr
Intrinsic "quote" returns its argument unevaluated.  It is used so frequently
it has an abbreviated form of a single apostrophe.
.Bd -literal -offset left
> (quote (a b c))
(a b c)
> '(a b c)
a
.Ed
.Pp
.Pp
.Pp
.Ss protect: (protect expr expr ...)
The "protect" intrinsic is analogous to unwind-protect in other lisp
dialects.  The function accepts one or more arguments and evaluates them in
sequence, but returns the value of evaluating the first argument.  The
arguments subsequent to the first are evaluated EVEN IF THE EVALUATION OF
THE FIRST IS INTERRUPTED by the interpreter encountering an error.
.Bd -literal -offset left
> (catch
>> (protect (throw 0)
>>> (print 'booger)
>>> (newline)))
booger
0
>
.Ed
.Pp
.Pp
.Pp
.Ss qquote: (qquote expr)
The "qquote" (quasiquote) macro accepts a list and returns it unchanged except
where sub-expressions have been "escaped" with comma characters.  Those
escaped sub-expressions are evaluated and the result of the evaluation
inserted into the template expression.  "qquote" aids in the composition of
macros.
.Pp
Some lisps define a read macro "`" as a short form for "qquote", but Munger
does not support this.  The commas escaping sub-expressions are separate
tokens in Munger, unlike in other lisps.  For example ,token is actually
parsed by Munger as two tokens, "," and "token".  This means ',token parses to
"(quote ,) token" and not "(quote ,token)".
.Pp
.Pp
.Pp
.Ss if: (if expr1 expr2 [exp3...])
Intrinsic "if" is a conditional.  It accepts two or three arguments, the first
being the test condition.  The test condition is evaluated, and if the result
is a true value (anything except 0, the empty string, or the empty list), the
second argument is evaluated and the result of that evaluation is returned.
Otherwise, if the test condition evaluated to a false value, and further
expressions are present after the second, those expressions are evaluated in
order and the result of the last expression evaluated is returned.  If only
two expressions are present in the original form, and the test condition
evaluates to a false value, the result of evaluating the test expression is
returned.
.Bd -literal -offset left
> (if (> 3 4) 'yes 'no)
no
> (if (> 3 4) 'yes)
0
.Ed
.Pp
.Pp
.Pp
.Ss and: (and expr1 [expr2 ...])
Intrinsic "and" accepts one or more  number of arguments, and evaluates them
from left to right until an argument evaluates to a "false" value (0, the
empty string, or the empty list), or the end of the arguments is reached. The
value of the last evaluation is returned.
.Bd -literal -offset left
> (and 1 'a 0)
0
> (and 1 'a "string")
1
.Ed
.Pp
.Pp
.Pp
.Ss or: (or expr1 [expr2 ...])
Intrinsic "or" accepts one or more arguments, and evaluates them from left to
right until an argument evaluates to a true value (anthing other than 0, the
empty string, or the empty list), or the end of the arguments is reached.  The
value of the last evaluation is returned.
.Bd -literal -offset left
> (or 1 0)
1
> (or 0 0)
0
.Ed
.Pp
.Pp
.Pp
.Ss list: (list expr1 [expr2 ...])
Intrinsic "list" accepts one or more arguments, evaluates them in order, and
returns a newly-constructed list containing of the result of each evaluation,
also in order.
.Bd -literal -offset left
> (list 'a 'b 'c) ; is equivalent to (cons 'a (cons 'b (cons 'c ())))
(a b c)
> (list '(a b c) 4 "hello")
((a b c) 4 "hello")
.Ed
.Pp
.Pp
.Pp
.Ss progn: (progn expr...)
Intrinsic "progn" accepts one or more arguments, and evaluates them in
order, returning the result of evaluating the last argument.  If no
arguments are passed to "progn" an error is generated which will stop
evaluation.
.Bd -literal -offset left
> (progn 
>> (set 'f (lambda (n) (+ n 1)))
>> (set 'x 1)
>> (f x))
2
.Ed
.Pp
.Pp
.Pp
.Ss prog1: (prog1 ...)
Library macro "prog1" accepts zero or more arguments, and evaluates them
in order, returning the result of evaluating the first argument.  If no
arguments are passed to the macro, it returns the empty list.
.Bd -literal -offset left
> (prog1
>> (+ 2 2)
>> (+ 3 3))
4
.Ed
.Pp
.Pp
.Pp
.Ss not: (not expr)
Intrinsic "not" returns 1 if its argument is the empty list, the empty string,
or zero, otherwise it returns 0.
.Bd -literal -offset left
> (not 0)
1
> (not "hello")
0
.Ed
.Pp
.Pp
.Pp
.Ss null: (null expr)
Library function "null" returns 1 if its argument is the empty list, otherwise
it returns 0.
.Bd -literal -offset left
> (null ())
1
> (null 'a)
0
.Ed
.Pp
.Pp
.Pp
.Ss pair: (pair expr)
Library function "pair" returns 1 if its argument is a non-empty list,
otherwise it returns 0.
.Bd -literal -offset left
> (pair '(a b c))
1
(pair ())
0
(pair 'a)
0
.Ed
.Pp
.Ss warn: (warn expr [expr...])
The "warn" intrinsic evaluates its arguments then writes the value of each
evaluation, and one final newline character to the standard error stream.  The
"warn" intrinsic always returns 1.
.Pp
.Pp
.Pp
.Ss setenv: (setenv expr1 expr2)
The "setenv" intrinsic sets the value of a named environment variable.  The
function accepts two arguments, which must both evaluate to strings.  The
first argument is the environment variable to set, and the second is the
new value to bind to the variable.  Any errors encountered will stop
evaluation.  Upon success, "setenv" returns 1.
.Pp
.Pp
.Pp
.Ss getenv: (getenv expr)
The "getenv" intrinsic looks up the value of an environment variable.  It
accepts one argument which must evaluate to a string.  If the environment
variable specified exists, a string is returned representing the variable's
value.  If the variable does not exist, 0 is returned.
.Bd -literal -offset left
> (getenv "HOME")
"/usr/home/jbailie"
> (getenv "foobar")
0
.Ed
.Pp
.Pp
.Pp
.Ss directory: (directory expr)
The "directory" intrinsic returns a list of strings representing the filenames
of the entires in a specified directory, or a string representing the error
encountered by the opendir() system call.  The function accepts one argument,
which must evaluate to a string, specifying the directory to list.
.Bd -literal -offset left
> (directory "/usr/local")
(".." "man" "bin" "share" "include" "lib" "etc" "info" "libexec" "sbin" "libdata")
> (directory "/foobar")
"No such file or directory"
.Ed
.Pp
.Pp
.Pp
.Ss chomp: (chomp expr)
The "chomp" intrinsic removes all contiguous terminating carriage return
and newline characters from a string.  The function accepts one argument
which must evaluate to a string, and ALWAYS returns a new string,
regardless of whether any terminators were removed from its argument or
not.
.Bd -literal -offset left
> (chomp (getline))
hello[return]
"hello"
.Pp
> (getline)
hello[return]
"hello
"
>
.Ed
.Pp
.SS chop: (chop expr)
.Pp
The "chop" intrinsic accepts one argument which must evaluate to a string, and
returns a new string with the same characters as the original string but with
the last character removed.  If the argument string is empty "chop" does
nothing.
.Bd -literal -offset left
> (chop "hello")
"hell"
> (chop "")
""
.Ed
.Pp
.Pp
.Pp
.Ss beep: (beep)
The "beep" intrinsic accepts no arguments and will cause the device connected
to standard output to beep, if it is capable of doing so.  1 is returned on
success.  Any error encountered stop evaluation.
.Pp
.Pp
.Pp
.Ss suspend: (suspend)
The "suspend" intrinsic accepts no arguments and causes the interpreter to
send a SIGSTOP to itself, suspending the interpreter process.  See your
shell's documentation on how to resume stopped jobs.
.Pp
.Pp
.Pp
.Ss pipe: (pipe expr1 expr2)
NOTE there are separate shortcut intrinsics to send or receive buffer
lines, to or from, inferior processes, or filter them through inferior
processes.  see the entries for the "input", "output", and "filter"
intrinsics for more details.
.Pp
the "pipe" intrinsic forks a specified process, piping one of the
interpreter's standard descriptors to the standard input or standard output of
the new process.  The function accepts two arguments, the first of which must
evaluate to 0, 1, or 2, and specifies the interpreter descriptor to be
redirected.  0 is the interpreter's standard input, 1 is the interpreter's
standard output, while 2 is the interpreter's standard error. if the user
wishes to read from the new process, 0 should be passed as the first argument,
but if the user wishes to write to the new process then 1 or 2 should be
passed as the first argument.  The second argument must evaluate to a string
specifying a command line to run.  The command is passed to the shell
(/bin/sh) for execution, and may contain any expression that program
understands.  All errors will stop evaluation.  Upon success "pipe" returns 1.
.Pp
Note that if the specified command cannot be found, or there is some other
error in the command line, the call to "pipe" will succeed, but the new
shell process will immediately exit.  If interpreter descriptor 0 were
redirected, then the next call to "getline" will return 0, indicating EOF,
but if descriptors 1 or 2 were redirected, the next "print" to the pipe
will cause the interpreter to receive a SIGPIPE, immediately terminating
it.
.Pp
Redirection is undone by the "resume" intrinsic.  It is not necessary to
undo a redirection, to perform another redirection.  Successive
invocations of "pipe" on the same descriptor may be made to create a
pipeline.  If interpretation returns to the toplevel, all redirections are
undone automatically.  See the entries for the "with_input_process" and
"with_output_process" macros, for a simple means of performing temporary
redirections.
.Pp
The following example is equivalent to entering:
.br
"jot 100 1 | sort -n | fmt"
.br
at the shell:
.Bd -literal -offset left
>(progn
>> (pipe 1 "fmt")
>> (pipe 1 "sort -n")
>> (for (a 100 1)
>>>   (print a)
>>>>  (newline)))
1
> 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25
26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47
48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69
70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91
92 93 94 95 96 97 98 99 100
.Ed
.Pp
Note the last line of output from the pipeline is a short line.  This means
that after all the data was written to the pipeline, the "fmt" utility
blocked, reading on its stdin, waiting for enough subsequent data to fill
another entire line.  It is only when the "progn" returned to toplevel and
the interpreter did an implicit "resume" on descriptor 1, that the pipeline
processes read EOF on the pipe, and "fmt" in turn, spit out the last line. 
The output of the pipeline and the interpreter process are mixed up because
they both have their standard outputs connected to the terminal.  The 1
after the lisp, is the return value of the lisp expression.  This is
followed by a the lisp prompt >, which is then followed by the output from
the pipeline.  It is necessary to wrap the whole example in a "progn" to
prevent the interpreter from returning to toplevel in-between
subexpressions, and undoing the redirections.  The child process spawned by
the second invocation of "pipe" inherited the redirection of stdout by the
first invocation of "pipe" which results in the creation of the pipeline.
Since the command argument to "pipe" is passed to the shell, we could have
used (pipe 1 "sort -n | fmt") instead and let the shell create the pipeline
for us.  Note that we must create the subprocesses of the pipeline in
reverse order if we are going to write to the pipeline, but in order if we
are going to read from the pipeline.
.Bd -literal -offset left
>(progn
>> (pipe 0 "jot 1 100")
>> (pipe 0 "fmt")
>> (while (set 'l (getline))
>>>   (print l)))
1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25
26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47
48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69
70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91
92 93 94 95 96 97 98 99 100
0
>
.Ed
.Pp
.Pp
.Pp
.Ss redirect: (redirect expr1 expr2 [expr3])
The "redirect" instrinsic redirects one of the three standard file descriptors
to a file.  The function accepts two or three arguments, the first of which
must evaluate to one of 0, 1, or 2, and specifies the descriptor to be
redirected.  The three possible values correspond to standard input, standard
output, and standard error, respectively.  The second argument must evaluate
to a string specifying the filename to be opened.  The third optional argument
cannot be present unless argument 1 evaluates to 1 or 2, and itself must
evaluate to a number.  If argument 3 evaluates to a non-zero value, it
indicates the contents of the file should be appended to.  If the third
argument is not present or is zero, and the first argument evaluated to 1 or
2, then the file will be created if it does not exist, or overwritten if it
does.  If the function is successful, the specified file descriptor will be
redirected onto the specified file, and 1 will be returned.  If an attempt is
made to redirect descriptor 0 to a non-existent file, -1 is returned.  If an
attempt is made to redirect descriptors 1 or 2, in appending mode, onto a
non-existent file, -1 is returned.  If an attempt is made to redirect any
descriptor to a file to which the user lacks the necessary permissions, -2 is
returned.  When redirecting stdin, the function attempts to acquire a shared
lock on the specified file, and when redirecting stdout and stderr, the
function attempts to acquire an exclusive lock on the file.  If a lock cannot
be obtained, the function returns -3.  If the function encounters any other
error from the open() system call, it will return a string describing the
error.  Errors from other sources will stop the interpreter.
.Pp
Redirection is undone by the "resume" intrinsic.  It is not necessary to undo
a redirection, to perform another redirection.  Successive invocations of
"redirect" may be made, and each redirection may be undone by calling "resume"
to reconnect the specified descriptor to the stream it was connected to
previously.  In other words, to undo all redirections, it is necessary to call
"resume" on a descriptor the same number of times one has called "redirect"
on the same descriptor.
.Pp
If the thread of execution returns to the toplevel read-eval-print loop
while one or more streams are redirected, all redirections will be undone
by the interpeter, with all three standard streams being reconnected to
the descriptors to which they were reconnected when the interpreted was
started.
.Pp
.Pp
.Pp
.Ss resume: (resume expr)
The "resume" intrinsic causes a redirected descriptor to be reconnected to
the device it was connected to before the last call to "redirect" or
"pipe".  The function accepts one argument which must evaluate to one of 0,
1, or 2, corresponding to the descriptor to be affected.  If the specifed
descriptor were not redirected, "resume" returns 0, otherwise it returns 1.
.Pp
.Pp
.Pp
.Ss with_input_file: (with_input_file expr1 expr2 ...)
The "with_input_file" macro temporarily redirects standard input onto a file.
The macro accepts two or more arguments, the first of which must evaluate to a
string specifying the file to read from, while the rest are expressions to be
evaluated while standard input is redirected.  The redirection is undone after
all the arguments have been evaluated.  No arguments are evaluated until after
the macro expands, in the calling scope.  The arguments following the first
argument will not be evaluated unless redirection has been successful.  The
macro returns the value of evaluating the last argument upon success,
otherwise it returns return value of the failed invocation of the "redirect"
intrinsic.
.Pp
The "resume" intrinsic is called by the macro to reconnect standard input to
the device it was connected to before the macro was invoked, after all the
body forms have been evaluated.
.Bd -literal -offset left
> (with_input_file "library.munger" 
>> (print (getline)))
; This file contains the lisp library code for the Munger interpreter. 
1
.Ed
.Pp
Note that calls to "with_input_file" may be nested.
.Bd -literal -offset left
> (with_input_file "library.munger" 
>> (print (getline)) 
>> (with_input_file "INSTALL" 
>>> (print (getline))) 
>> (print (getline))) 
; This file contains the lisp library code for the Munger interpreter.
Munger Installation
; Copyright (c) 2001, 2002, 2003, James Bailie <jimmy@jamesbailie.com>.
1
>
.Ed
.Pp
.Pp
.Pp
.Ss with_output_file: (with_output_file expr1 expr2 ...)
The "with_output_file" macro behaves similarly to the "with_input_file"
macro except that "with_output_file" redirects standard output instead of
standard input.  The specified file will be overwritten, if it already
exists.
.Bd -literal -offset left
> (with_output_file "tmp" (print "hello") (newline))
1
> (with_input_file "tmp" (print (getline)))
hello
1
.Ed
.Pp
.Pp
.Pp
.Ss with_output_file_appending: (with_output_file_appending expr1 expr2 ...)
The "with_output_file_appending" macro behaves similarly to the
"with_output_file" macro, except that the specified file will be opened for
appending.
.Pp
.Pp
.Pp
.Ss with_input_process:
.Ss with_output_process:
These two macros behave similary to the "with_input_file" and
"with_output_file" macros, the only difference being the first argument
specifies a program for the appropriate descriptor to be piped to or from,
instead of a file.  The first argument is passed to /bin/sh for processing and
may therefore contain any commands that program understands.  Read the entries
for with_input_file and with_output_file for more information.
.Bd -literal -offset left
> (with_input_process "ls"
   (while (set 'l (getline))
      (print l)))
INSTALL
LICENSE
Makefile
README
cat.munger
cgi.munger
cgi.munger
echo.fcgi
err.munger
grep.munger
intrinsics.c
library.munger
lisp.c
lisp.h
munger.man
tmltags.munger
transform.munger
.Ed
.Pp
.Pp
.Pp
.Ss foreach_line: (foreach_line expr)
The "foreach_line" library macro is used to create filter-like programs.  The
function accepts one argument, which must evaluate to a monadic function.  The
monadic function is called repeatedly with each line of input.  The input
lines are taken from the files specified on the command line, or if no files
are specified, from the standard input.  Note the function passed to the macro
can be an intrinsic which accepts variable length argument lists, but is happy
with just one argument, such as the "print" instrinsic.
.Pp
The argument pointer must be positioned so that the next call to (next) will
return the first of the command line arguments the user wishes processed, if
any.  A script would normally process any option arguments, advancing the
argument pointer as it did so, and then call foreach_line.  The invocation of
foreach_line returns when it has processed all the specified files, or when it
encounters EOF on standard input.
.Bd -literal -offset left
; Skip over the interpreter name, so that the argument pointer is now
; pointing to the script name.  The next invocation of (next) will 
; either return the next argument, or 0.

(next)
(foreach_line (lambda (line) [do something with line] ))
(exit 0)
.Ed
.Pp
.Pp
.Pp
.Ss foreach_line_callback: (foreach_line_callback expr1 expr2 (expr3))
The "foreach_line_callback" macro functions similarly to the "foreach_line"
macro, but is intended for use by filters which need to reset state variables
when the input source changes, and/or which wish to output the results of
processing the just-completed input source.  The macro accepts one or two more
arguments than "foreach_line".  The second argument must evaluate to a
function, which itself accepts no arguments, which will be called at the end
of processing each input source.  The third argument is optional, and if
present, is treated as a logical flag which specifies whether the callback
should be invoked before or after the argument pointer is advanced.  In
Munger, zero, the empty list, and the empty string are all logical "false"
values; all other objects are considered "true".  A "true" third argument
causes the callback to be invoked after the argument pointer has been
advanced.  Ommitting the third argument, or setting it to one of the three
"false" values, causes the callback to be invoked before the argument pointer
is advanced.  Within the callback function, invoking (current) will return the
command-line argument the argument pointer is currently pointing at.
.Pp
This means if you wish to access the name of the file which has just been
processed in the callback, no third argument, or a "false" third argument
should be passed to the macro, but if you wish to access the name of the new
input source about to be processed, a "true" third value should be passed as
the third argument.  If the currently-running script has not been presented
with any command-line arguments, then "foreach_line_callback" will read data
from the standard input, and then invoke the callback function once after all
data has been processed.  The callback function should be crafted with this
possibility in mind.
.Pp
The "grep.munger" example program in (libdir) demonstrates using the third
argument.  The following script does not supply a third argument, and
simulates wc -l:
.Bd -literal -offset left
(next)
(set 'script (current))
(set 'count 0)
(set 'total 0)

(set 'callback
   (lambda ()

      (let ((name (if (eq script (current)) "stdin" (current))))

         (print count " " name)
         (newline)
         (set 'total (+ total count))
         (set 'count 0))))

(foreach_line_callback (lambda (x) (inc count)) callback)
(print total  " total")
(newline)
(exit 0)
.Ed
.Pp
.Pp
.Pp
.Ss unlink: (unlink expr)
The "unlink" intrinsic removes a file from a directory.  It accepts one
argument which must evaluate to a string specifying the filename to remove.
The function returns 1 upon success, or a string specfiying the error
encountered upon failure.
.Pp
.Pp
.Pp
.Ss realpath: (realpath expr)
The "realpath" intrinsic resolves all symbolic links, extra / characters,
and references to . and .. in a specified pathname.  If the filename begins
with a tilde (~), the function will attempt to expand the tilde as csh or
bash does to reference a home directory.  The Function accepts one
argument, which must evaluate to a string, and returns a string.  If the
pathname is fully-qualified, either explicitly or with use of "~", and the
file does not exist, an error is generated which will stop evaluation.
.Bd -literal -offset left
> (realpath ".")
"/usr/home/jbailie/src/munger-4.81"

> (realpath "~/foobar")
"/usr/home/jbailie/foobar"

; Tilde-expansion does not occur if the tilde-expression does not reference
; a valid, readable, home directory.  Here the tilde is considered to be
; the first character of a relative filename, because there is no 
; ~nobody home directory on my system.

> (realpath "~nobody/foobar")
"/usr/home/jbailie/src/munger-4.81/~nobody/foobar"
.Ed
.Pp
.Pp
.Pp
.Ss access: (access expr1 expr2)
The "access" intrinsic determines whether or not the real user id the
interpreter is running as, has access to a specified file.  The function
accepts two arguments, the first of which must evaluate to a string and
specifies the filename to check access to, while the second argument must
evalaute to a number and be one of 0, 1, or 2.  A second argument of 0
instructs the function to check for read access to the specified file.  A
second argument of 1 instructs the function to check for write access to
the file, and a second argument of 2 instructs the function to check for
executable access to the file.  The function will return 1 to indicate
access is allowed, or 0 otherwise.
.Pp
.Pp
.Pp
.Ss mkdir: (mkdir expr)
The "mkdir" intrinsic accepts one argument, which must evaluate to a
string, and attempts to create a directory with the same name as the
string.  Upon success, "mkdir" returns 1; otherwise it returns a string
describing the error encountered.  If successful, the newly-created
directory will have permissions 755 (possibly modified by the current
umask).
.Pp
.Pp
.Pp
.Ss rmdir: (unlink expr)
The "rmdir" intrinsic removes a directory from the filesystem.  It accepts
one argument which must evaluate to a string specifying the directory name,
and returns 1 upon success, or a string specifying the error encountered upon
failure.
.Pp
.Pp
.Pp
.Ss words: (words)
The "words" intrinsic accepts no arguments and returns the number of words
in the buffer.
.Pp
.Pp
.Pp
.Ss time: (time)
The "time" intrinsic accepts no arguments and returns a string representing
an integer representing the number of seconds since 00:00 Jan 1, 1970 (the
UNIX epoch).  It is not possible to return the integer value directly, as
it overflows the 31 bits of the fixnum representation on 32-bit machines.
Note that one may compare strings representing integers with the "strcmp"
intrinsic if they are the same length, and get the same result as a
direct comparison of the two numbers would return.  If the strings are of
different lengths, than the shorter one will represent the smaller number
(Munger suppresses leading zeroes).
.Pp
.Pp
.Pp
.Ss timediff: (timediff expr1 expr2)
The "timediff" intrinsic accepts two arguments which both must evaluate to
strings specifying a time value expressed in seconds from the UNIX epoch
(00:00 Jan 1, 1970), such as returned by the "time" intrinsic, and returns
the difference in seconds between the two times, as a number.  If the time
difference cannot be expressed within the word-size of the machine the
Munger interpreter is running on, minus one bit (31 bits on a 32-bit
machine), then fixnum wrap-around will occur and the returned value will
not be accurate.  See the section at the top of this document, titled
NUMBERS, for information on fixnum wrap-around.
.Pp
.Pp
.Pp
.Ss timethen: (timethen expr)
The "timethen" intrinsic accepts one argument which must evaluate to a
fixnum and returns a string representing the current time in seconds from
the UNIX epoch (00:00 Jan 1, 1970), offset by that many seconds.  This
function can be used to determine the UNIX time value for a time in the
future or the past.  For example, the following code returns the UNIX time
value for a time 1 hour in the past:  
.Bd -literal -offset left
> (timethen -3600)
"1124553616"
.Ed
.Pp
.Pp
.Pp
.Ss date2days: (date2days expr1 expr2 expr3)
The "date2days" intrinsic accepts three arguments, each of which must
evaluate to a fixnum, specifying a year, month, and a day of the month, and
returns a fixnum representing the number of days this date is removed from
March 1st, 1 BCE.  This value can be useful in performing calendar
arithmentic.  The first number indicates the year and must be greater than
or equal to 0, (1 BCE).  The second number indicates the month and must be
in the range 1-12.  The third number indicates the day of the month and
must be in the range 1-31, or 1-30, depending on the month specified.
.Pp
.Pp
.Pp
.Ss days2date: (days2date expr)
The "days2date" intrinsic accepts a day number fixnum returned by
"date2days" and converts it into a list of three fixnums representing that
date.  The first number indicates the year and will be greater than or
equal to 0, (1 BCE).  The second number indicates the month and will be in
the range 1-12.  The third number indicates the day of the month and will
be in the range 1-31.
.Pp
.Pp
.Pp
.Ss week: (week expr)
The "week" intrinsic accepts one argument which must evaluate to a fixnum,
representing a day number as returned by "date2days", and returns a
two-element list of fixnums representing the year and the week number in
that year, respectively, in which the specified day occurs.
.Pp
.Pp
.Pp
.Ss weekday: (weekday expr)
The "weekday" accepts one argument which must evaluate to a day number,
such as returned by "date2days", and returns a two-element list describing
the day of the week of the specified day.  The first returned element is a
fixnum in the range 0-6, each value of which corresponds to a weekday in
the range Sunday-Saturday, respectively.  The second element is a string
representing the name of the weekday, in English.
.Pp
.Pp
.Pp
.Ss month: (month expr)
The "month" intrinsic accepts a fixnum argument specifying the numerical
value of a month (1-12), as used by the "date2days", "days2date", and
"localtime" intrinsics, and returns a string representing the name of the
month in English.
.Pp
.Pp
.Pp
.Ss localtime: (localtime expr)
The "localtime" intrinsic accepts a string representing a UNIX time value,
such as returned by the "time" intrinsic, and returns a six-element list of
fixnums representing the date of the specified time.  The first returned
element is the year.  The second is the month.  The third is the day of the
month.  The fourth is the hour.  The fifth is the minutes.  The sixth is
the seconds.
.Pp
.Pp
.Pp
.Ss date2time: (date2time expr1 expr2 expr3...)
The "date2time" intrinsic converts a list of fixnums representing a date
on or after midnight January 1st, 1970, and returns a fixnum representing
the number of seconds from the UNIX epoch (midnight, January 1st, 1970).
See the entry for the "time" intrinsic for more information on UNIX time
values.  The funtion accepts three, four, five, or six arguments.  The
first three arguments must be present, and represent the year (1970-), the
month (1-12), and the day of the month (1-31), respectively.  The fourth,
fifth, and sixth optional arguments represent the hour, minute, and seconds
values of the date, respectively.  Omitted optional arguments defualt to 0.
The presence of an optional argument implies the presence of all preceding
optional arguments.  This means if you include a value for minutes, you
must include a value for hours as well, and if you include a value for
seconds, you must include values for hours and minutes as well.
.Pp
.Pp
.Pp
.Ss random: (random expr)
The "random" intrinsic accepts one argument which must evaluate to a positive
number, and returns a random integer where 0 < returned-value <
evaluated-argument.  The interpreter calls srandomdev() at startup to
initialize the random number generator, and random() to generate random
numbers.  Random numbers are then scaled to be within the requested range
by computing:  evaluated-argument * random-number / RAND_MAX.
.Pp
.Pp
.Pp
.Ss date: (date [expr])
The "date" intrinsic accepts one optional argument and returns a string
representing the time and date.  The optional argument, if present, must
evaluate to a number.  Invoking "date" with no arguments is the same as
invoking it with an argument of 0.  When invoked with an argument of 0, the
function returns a representation of the time and date expressed in terms of
the local timezone.  When invoked with a non-zero argument, the function
returns a representation of the time and date expressed as Greenwich Mean
Time.  The format of the date string conforms to RFC 2616 (HTTP 1.1).
.Bd -literal -offset left
> (date)
"Fri, 04 Oct 2002 12:06:11 EDT"
> (date 1)
"Fri, 04 Oct 2002 16:06:13 GMT"
.Ed
.Pp
.Pp
.Pp
.Ss print: (print expr...)
Intrinsic "print" accepts one or more arguments, evaluates them, and prints
the each evaluated argument, in order,  to the standard output stream.  If any
of the arguments evaluate to strings, the contents of the strings are printed
without the surrounding quotes.  The "print" intrinsic always returns 1.
.Bd -literal -offset left
> (print "hello there")
hello there
1
.Pp
> (set 'f '(a b c))
(a b c)
1
> (print f)
(a b c)
.Pp
> (print 'hello)
hello1
.Pp
> (progn
>> (print 'hello)
>> (newline))
hello
1
.Ed
.Pp
.SS newline: (newline)
.Pp
The "newline" intrinsic outputs a newline character (ASCII 10) to the
standard output.
.Pp
.Pp
.Pp
.Ss load: (load expr)
Intrinsic "load" reads lisp from the file specified by its lone argument,
which must evaluate to a string.  The function returns 1 upon success, 
while any errors encountered opening the file, or evaluating its content
will stop evaluation.  "load" expects its argument to be a regular file,
but if it is a non-seekable entity such as a named pipe, it will work as
expected, but any recursive invocation will cause buffered data from the
non-seekable entity to be lost.
.Pp
.Pp
.Pp
.Ss getline: (getline [ expr1 [expr2] ] )
Intrinsic "getline" reads a line from standard input and returns it as a
string, including the terminating newline.  The first optional argument, if
present, must evaluate to a string, while the second optional argument, if
present, must evaluate to a whole number.  Upon reading EOF, "getline"
returns 0.
.Pp
IF THE STANDARD INPUT IS NOT A TERMINAL DEVICE, BOTH ARGUMENTS ARE IGNORED
AND THE BEHAVIOUR IN THE REST OF THIS DESCRIPTION DOES NOT OCCUR.
.Pp
If standard input is a terminal device, then the terminal is put into "raw"
mode, with the interpreter simulating the standard UNIX terminal line
discipline (ctrl-h => backspace; ctrl-w => werase; ctrl-u => clear).  If
ctrl-d (EOF) is encountered as the first character of input, "getline" returns
the number zero, otherwise EOF is ignored, and getline continues to accumulate
characters until it a carriage-return or a newline is input.  All other
control characters are discarded.  A carriage return character is converted
into a newline, so lines input from terminal devices will always be
newline-terminated.  Note this does not happen when standard input is not
connected to a terminal device.
.Pp
All printable characters typed by the user are echoed to stdout if stdout
is a terminal device.  If is not, then an attempt is made to open /dev/tty
for writing.  If this is successful, the characters are echoed there, but
if this is not successful, no echoing takes place.  The purpose of this is
to attempt to mimic the behavior of the terminal driver inside munger.  For
example, if one runs cat from the shell with its stdout redirected to a
file, the terminal driver will echo characters the user types, but if
munger did this, it would echo those characters into the file stdout is
connected to.
.Pp
The function accepts two optional arguments, the first of which, if
present, must evaluate to a string, and is printed as a prompt to the user.
If the second optional argument is also present, it must evaluate to a
number and specifies the location of tabstops in the onscreen echoed
string.  For example a value of 4, will cause tabstops to appear to the
user to be set every four characters.  If the second argument is not
present, tabstops default to every 8 characters.  If the second argument is
0, then the input of a tab character will trigger filename completion.
Note that in this case, tabs cannot be entered by the user.  Pass the empty
string as the first argument, if you wish to submit a second argument to
the function, but do not wish it to print a prompt.
.Bd -literal -offset left
> (set 'input (chomp (getline)))
   You type: hello[enter]
> "hello
"
.Ed
.Pp
.Pp
.Pp
.Ss stringp: (stringp expr)
Intrinsic "stringp" returns 1 if its lone argument evaluates to a string,
otherwise it returns 0.
.Bd -literal -offset left
> (stringp "0")
1
> (stringp 0)
0
.Ed
.Pp
.Pp
.Pp
.Ss numberp: (numberp expr)
Intrinsic "numberp" returns 1 if its lone argument evaluates to a number,
otherwise, it returns 0.
.Bd -literal -offset left
> (numberp 0)
1
> (numberp "0")
0
.Ed
.Pp
.Pp
.Pp
.Ss symbolp: (symbolp expr)
Intrinsic "symbolp" returns 1 if its lone argument evaluates to a symbol,
otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss regexpp: (regexpp expr)
Intrinsic "regexpp" returns 1 if its lone argument evaluates to a compiled
regular expression object, otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss tablep: (tablep expr)
Intrinsic "tablep" returns 1 if its lone argument evaluates to a table,
otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss stackp: (stackp expr)
Intrinsic "stackp" returns 1 if its lone argument evaluates to a stack,
otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss intrinsicp: (intrinsicp expr)
Intrinsic "intrinsicp" returns 1 if its lone argument evaluates to an
intrinsic function, otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss closurep: (closurep expr)
Intrinsic "closurep" returns 1 if its lone argument evaluates to a closure,
otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss macrop: (macrop expr)
Intrinsic "macrop" returns 1 if its lone argument evaluates to a macro
closure, otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss recordp: (recordp expr)
Intrinsic "recordp" returns 1 if its lone argument evaluates to a record,
otherwise it returns 0.
.Pp
.Pp
.Pp
.Ss dbp: (dbp expr)
Intrinsic "dbp" returns 1 if its lone argument evaluates to a database object,
otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss regcomp: (regcomp expr1)
The "regcomp" intrinsic accepts a single argument which must evaluate to a
string, and which is interpreted as a regular expression to be compiled into
a compiled regular expression object.  The function returns the compiled
regular expression object if the compilation is succesful, or a string
containing an error message, otherwise.  Regular expression objects are
constants which evaluate to themselves.
.Pp
The "regcomp" intrinsic interprets the two-character escape-sequences \\t,
and \\b, to represent the tab and space characters respectively, allowing
one to specify regular expressions containing whitespace, without using
whitespace.  This can sometimes be convenient.  Also the zero width
assertions \\< and \\> are recognized as abbreviations of the POSIX bracket
expressions [[:<:]] and [[:>:]], and match the null string at the beginning
and end of a word, respectively.
.Pp
.Pp
.Pp
.Ss substitute: (substitute expr1 expr2 expr3 [expr4])
Intrinsic "substitute" performs search and replace operations on strings
using regular expressions to specify the matches.  The function uses a
template string to specify the replacement text, in a manner similar to 
that of the substitute commands of ed and sed, where the template may refer
to the text of the match, and the text matched by parenthesized
subexpressions.  There is also a "replace" library function available,
described elsewhere in this document, which allows the replacement
expression to be a snippet of lisp, which is evaluated upon each match to
dynamically generate the replacement text.
.Pp
The "substitute" function takes three or four arguments, and returns a new
string, incorporating the appropriate replacements.  The first argument
must evaluate to a compiled regular expression.  See the "regcomp"
intrinsic for more information on compiling regular expressions.  The
second and third arguments must evaluate to strings, and are interpreted as
the replacement string, and the string to search for matches in,
respectively.  The fourth optional argument, if present, must evaluate to a
number, which indicates the number of matches which should be replaced. A
value of 0 indicates the substitution should replace all the matches of the
pattern in the text of the third argument.  The absence of the fourth
argument is the same as having a fourth argument of 1, which is to say,
only the first match will be replaced.
.Pp
The text matched by the first ten subexpressions in the regular expression may
be inserted into the replacement string by including an escape sequence of the
form \\[0-9] in the replacement text where one wishes the matched text to
appear. The first subexpression is referred to by \\1 and the tenth
subexpression is referred to by \\0.  The text matched by the entire regular
expression may be inserted into the replacement text by inserting \\& into the
replacment string at the desired location.
.Bd -literal -offset left
> (substitute (regcomp "string") "booger" "string string string string" 2)
"booger booger string string"
.Pp
> (substitute (regcomp "[a-z]+") "{\\&}"  "one two three" 0) 
"{one} {two} {three}"
.Pp
> (substitute (regcomp "([a-zA-Z]+) ([a-zA-Z]+)") "\\2 \\1"  
>> "is This sentence a."  0) 
"This is a sentence."
.Ed
.Pp
The "substitute" intrinsic interprets the two-character escape-sequences
\\t, and \\b, to represent the tab and space characters respectively,
allowing one to specify whitespace in the replacement string, without
using whitespace.  This can sometimes be convenient.  Five other escape
sequences may be embedded in the replacement string to control the case of
portions of the returned string.  These escape sequences work for strings
of ASCII alphanumeric characters, only:
.Bd -literal -offset left
\\U Turns on conversion to uppercase.
\\u Turns on conversion to uppercase for the next character only.
\\L Turns on conversion to lowercase.
\\l Turns on conversion to lowercase for the next character only.
\\e Turns off \\U or \\L.
.Pp
> (substitute (regcomp "foo") "\\U\\&\\e" "foobar") 
"FOObar"
.Ed
.Pp
The effects of \\U and \\L extend beyond the replacement string if they are
not terminated:
.Bd -literal -offset left
> (substitute (regcomp "foo") "\\U\\&"  "foobar") 
"FOOBAR"
.Ed
.Pp
\\U and \\L override each other:
.Bd -literal -offset left
> (substitute (regcomp "foo") "\\U\\&\\L" "foobar FUNBAG") 
"FOObar funbag"
.Ed
.Pp
.Pp
.Pp
.Ss match: (match expr1 expr2)
The "match" intrinsic matches a regular expression against a string, and
returns a two element list if the regular expression finds a match within
the string, or the empty list otherwise.  The function accepts two
arguments, the first of which must evaluate to a compiled regular
expression object, while the second must evaluate to the string to match
the regular expression against.  The two-element list returned upon
success, consists of the character indices of the starting and ending
locations of the matched text.  The second index is the start of the text
following the match.  These two indices may be used in conjunction with the
"substring" intrinsic, to extract the text before, after, or containing the
match:
.Bd -literal -offset left
> (set 'rx (regcomp "foobar"))
<REGEXP#3>
> (set 's "---foobar---")
"---foobar---"
> (match rx s)
(3 9)
> (substring s 0 3) ; text before the match
"---"
> (substring s 3 (- 9 3)) ; text of the match
"foobar"
> (substring s 9 0) ; text after the match
"---"
.Ed
.Pp
Note that if the first returned index were 0, the first invocation of
"substring" above would have returned the whole string, because passing a
third length argument of 0 to "substring" means "to the end of the string."
This should not be a problem, since a first returned index of 0 indicates
there is no text before the match.  The programmer can check for this
situation before invoking "substring".
.Pp
The "matches" intrinsic, described directly below, can be used to extract
the text of a match and any matching parenthesized subexpressions; while
"match" itself, is intended to be used in situations where it is only
necessary to know whether a match occurred or not, or if the text not
matched needs to be accessed, or if the location of the match is required.
.Pp
.Pp
.Pp
.Ss matches: (matches expr1 expr2)
The "matches" intrinsic accepts the same arguments as the "match" intrinsic,
but returns a list, which if non-empty, describes the text matched.
.Pp
If no match were found, the list will be empty.  Otherwise, a list of twenty
strings will be returned.  The first string of the list will be the text
matched by the entire regular expression, while the subsequent elements of
the list will be the text matched by the first nineteen parenthesized
subexpressions in the regular expression.  If nineteen subexpressions are
not present in the regular exression, empty strings will be returned for
the missing subexpressions.  If a subexpression fails to match, an empty
string will be returned in the corresponding position of the list.
.Pp
.Bd -literal -offset left
> (matches (regcomp "[0-9]+") "I have 22 figurines.") 
("22" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "") 
> (matches (regcomp "([0-9])+") "12345") 
("12345"  "5" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "") 
> (matches (regcomp "([0-9]+)") "12345") 
("12345"  "12345" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" ")
.Ed
.Pp
If one is using a regular expression with a lot of parenthesized
subexpressions, it may be more convenient to assign the results of invoking
"matches" to a stack.  The final zero in the example below is the return value
of the "print" intrinsic.
.Bd -literal -offset left
> (set 'rx (regcomp "(.)(.)(.)(.)(.*)"))
<REGEX1>
> (when (used (set 's (assign (stack) (matches rx "fooobar"))))
>> (for (a 0 19)
>>> (print (index s a) ":")))
foobar:f:o:o:o:bar:::::::::::::::1
.Ed
.Pp
.Pp
.Pp
.Ss split: (split expr1 expr2 [expr3])
The "split" intrinsic is used to break up a string into substrings at
delimiter characters.  It accepts either two or three arguments.  Both of
the first two arguments must evaluate to strings.  The first is interpreted
as a set of delimiter characters which specify where the second argument
should be split.  Where a character from the delimiter string occurs in the
second argument, it is consumed, breaking up the second argument into
pieces.  The pieces are returned as a list of strings.  The optional third
argument must evaluate to a number, and specifies a limit on the number of
pieces the second argument is to be split into.  If the first argument is
the empty string, then the second argument is split up
character-by-character.  The original string is unchanged.  The "split"
intrinsic will return empty strings for empty fields in the original
string.  Delimiter characters occurring adjacent to each other, or at the
very start or very end of the second argument, will be recognized as empty
fields.  If none of the delimiter characters can be found in the string,
then the returned list will contain only the original string.
.Bd -literal -offset left
> (split " "  "Lisp is an applicative language.") 
("Lisp" "is" "an" "applicative" "language.") 
> (split " "  "Lisp is an applicative language.", 3) 
("Lisp" "is" "an applicative language.") 
> (split "" "Lisp is an applicative language", 3) 
("L" "i" "sp is an applicative language")
> (split ":" ":a:b:c:")
("" "a" "b" "c" "")
.Ed
.Pp
.Pp
.Pp
.Ss split_rx: (split_rx expr1 expr2 [expr3])
The "split_rx" library function is used to break up a string into
substrings using a regular expression to specify delimiter substrings.  It
works analogously to the "split" intrinsic, except the first argument must
evaluate to a compiled regular expression object.  As with "split", the
second argument must evaluate to a string, and the third argument, if
present limits the number pieces the second argument will be broken into.
The supplied regular expression is matched against the second argument,
then the text of matches is consumed to split the second argument into
pieces, which are gathered into a list and returned.  The function returns
a list containing only the original second argument, if no match on the
regular expression can be found.  Empty fields cannot be detected by
split_rx.
.Bd -literal -offset left
> (setq rx (regcomp "[\\b\\t]+"))
<REGEXP#34>
> (split_rx rx "foobar    tooley marzipan  loopy    ")
("foobar" "tooley" "marzipan" "loopy")
.Ed
.Pp
.Pp
.Pp
.Ss rootname: (rootname expr)
The "rootname" library function accepts one argument which must evaluate to
a string, and returns the portion of that string, without the last trailing
filename suffix.  If the argument does not have a suffix the original
string is returned.
.Bd -literal -offset left 
> (rootname "foobar.tar.gz")
"foobar.tar"
> (rootname (rootname "foobar.tar.gz"))
"foobar"
.Ed
.Pp
.Pp
.Pp
.Ss suffix: (suffix expr)
The "suffix" library function accepts one argument which must evaluate to a
string and returns the last filename suffix contained in the string.  If
the string does not end with a filename suffix, the original string is
returned.
.Bd -literal -offset left
> (suffix "foobar.tar.gz")
".gz"
> (suffix (rootname "foobar.tar.gz"))
".tar"
.Ed
.Pp
.Pp
.Pp
.Ss tokenize: (tokenize expr)
The "tokenize" library function accepts one argument which must evaluate to
a string, and returns a list consisting of all the tokens embedded in the
string composed of contiguous non-whitespace characters.  The function
returns a list containing only the original argument string if it contains
no whitespace.
.Bd -literal -offset left
> (tokenize "  foobar tooley   hot-cha-cha!")
("foobar" "tooley" "hot-cha-cha!")
.Ed
.Pp
.Pp
.Pp
.Ss replace: (replace expr1 expr2 expr3 [expr4])
The "replace" library macro behaves similarly to the "substitute"
intrinsic, except that the replacement expression will be executed once
for every match in the string.  This allows the replacement string to be
dynamically created for each match.  The function accepts three or four
arguments.  The first argument must evaluate to a compiled regular
expression object.  The second argument is the expression to create the
replacement string and is evaluated in a lexical context where the symbols
m0...m19 are bound to the text of the full match, and the text matched by
the first nineteen parenthesized subexpressions, respectively.  Note that
the resulting string this argument produces when evaluated will undergo
some escape-processing, because it is ultimately used as the replacement
pattern for a call to the "substitute" intrinsic.  See the entry describing
that function, elsewhere in this document, for details on the escape
sequences recognized.  Note also, one can simply pass a string as the
second argument, but in that case it is much more efficient to use
"substitute" directly.  The third argument must evaluate to the string the
regular expression is to be matched against.  The fourth optional argument,
if present is a repeat count, and can be used to limit the number of
replacements made in the string.  Unlike the "substitute" intrinsic,
"replace" interprets a repeat count of 0 as a request to inhibit any
substitutions from being performed.  If the repeat count is not present,
all matches in the string will undergo replacement.  If no match is found,
the function returns the original string, otherwise it returns a new
string containing the specified substitutions.
.Pp
The following example uses "replace" to decode x-www-form-url-encoded
characters in a specified string.
.Bd -literal -offset left
> (setq str "foobar+tooley%7Efunbag%24%25%26")
"foobar+tooley%7Efunbag%24%25%26"

> (setq r0 (regcomp "%([0-9A-Fa-f])([0-9A-Fa-f])"))
<REGEXP#10>
> (setq r1 (regcomp "\\+"))
<REGEXP#11>

> (setq hex2decimal
>  (lambda (x)
>>    (let ((c (code (upcase x 0))))
>>>      (if (and (>= c 65) (<= c 70))
>>>>        (- c 55)
>>>>        (digitize x)))))
<CLOSURE#120>

; Decode encoded spaces.

> (setq str (substitute r1 " " str 0))
"foobar tooley%7Efunbag%24%25%26"

; Decoded hex encodings.

>(setq str 
>> (replace r0 
>>>        (char (+ (* 16 (hex2decimal m1)) (hex2decimal m2)))
>>>        str))
"foobar tooley~funbag$%&"
.Ed
.Pp
.Pp
.Pp
.Ss concat: (concat expr ...)
The "concat" macro is used to concatenate strings.  The macro accepts one
or more arguments which must all evaluate to strings, or to arbitrarily
nested lists containing only strings, and returns a single string
consisting of all the argument strings concatenated.
.Bd -literal -offset left
> (concat "foo" "bar")
"foobar"
> (concat '("foo" "bar"))
"foobar"
.Ed
.Pp
.Pp
.Pp
.Ss explode: (explode expr)
The "explode" macro accepts one argument which must evaluate to a string
and returns a list of one-character strings, corresponding to each character
in the original string.
.Bd -literal -offset left
> (explode "foobar")
("f" "o" "o" "b" "a" "r")
.Ed
.Pp
.Pp
.Pp
.Ss join: (join expr1 expr2 expr3...)
The "join" intrinsic is used to concatenate strings with delimiters.  It
accepts three or more arguments, which must all evaluate to strings, or to
arbitrarily-nested lists containing only strings.  The first argument is
sandwiched between the strings of the remaining arguments to make a new
string, which is returned.
.Bd -literal -offset left
> (join " "  "This" "will" "be" "a" "sentence.") 
"This will be a sentence."  
> (join ""  "this" "will" "be" "run" "together")
"thiswillberuntogether"
> (join ":" '("a" ("b" ("c") "d") "e"))
"a:b:c:d:e"
.Ed
.Pp
.Pp
.Pp
.Ss length: (length expr)
The "length" intrinsic accepts one argument which must evaluate to a string or
a list, and returns either the number of characters in the argument, if the
argument is a string, or the number of elements in the argument, if the
argument is a list.
.Bd -literal -offset left
> (length "fiver") 
5 
> (length '(a b c)) 
3
.Ed
.Pp
.Pp
.Pp
.Ss sort: (sort [expr...])
The "sort" intrinsic accepts any number of arguments and sorts them in
ascending order if they all evaluate to either strings or numbers.  Invoking
"sort" without arguments returns the empty list.  When sorting strings, case
is ignored.
.Bd -literal -offset left
> (sort "D" "c" "B" "a")
("a" "B" "c" "D")
> (sort 3 2 1)
(3 2 1)
.Ed
.Pp
.Pp
.Pp
.Ss sortlist: (sort expr)
The "sortlist" intrinsic accepts one argument which must evaluate to a list
whose elements must be either all numbers or all strings.  The function
returns a new list whose members are the members of the original list sorted
into ascending order.  When sorting strings, case is ignored.
.Bd -literal -offset left
> (sortlist '("c" "b" "a")) 
("a" "b" "c") 
> (sortlist '(0 -23 1)) 
(-23 0 1)
.Ed
.Pp
.Pp
.Pp
.Ss sortcar: (sortcar expr)
The "sortcar" intrinsic accepts one argument which must evaluate to a list and
whose elements must all be lists themselves, whose first elements must be
either all strings or all numbers.  The function returns a new list containing
the elements of the original list, sorted using the first elements of the
sublists as the sort key.  An example will make the meaning of this convoluted
description clear:  
.Bd -literal -offset left
> (sortcar '(("Z" f f) ("B" f f) ("C" f f)))
(("B" f f)("C" f f)("Z" f f))
.Ed
.Pp
"sortcar" is useful for sorting aggregate types.  The following example
sorts a stack-of-stacks, using the second element of each substack as the
sort key.  Perl users may recognize this as an example of an idiom that
community has come to call "The Schwartzian Transform," but of course, this
sort of thing is pure lisp.
.Bd -literal -offset left
> (set 's (stack 3))
<STACK1>
> (store s 0 (assign (stack) '(f 2 f)))
<STACK2>
> (store s 1 (assign (stack) '(f 1 f)))
<STACK3>
> (store s 2 (assign (stack) '(f 0 f)))
<STACK4>
> (flatten s)
(<STACK2> <STACK3> <STACK4>)
> (assign s (mapcar (lambda (x) (cadr x))
                    (sortcar (mapcar (lambda (x) (list (index x 1) x)) 
                                     (flatten s)))))
<STACK1>
> (flatten s)
(<STACK4> <STACK3> <STACK2>)
.Ed
.Pp
.Pp
.Pp
.Ss until: (until expr1 [expr2 ...])
.Ss while: (while expr1 [expr2 ...])
The "while" intrinsic is a looping construct.  It accepts one or more
arguments, the first of which is the test condition.  If it evaluates to a
true value (anything other than 0, the empty string, or the empty list), the
rest of the arguments are evaluated in order.  This process is repeated until
the first argument no longer evaluates to a true value, when the loop stops
without evaluation of the other arguments.  Therefore a while-loop will
execute zero or more times.  The "while" intrinsic returns the value of the
failed test expression.
.Pp
The "until" intrinsic has similar behavior to the "while" intrinsic, except
the logic of the test is inverted, which is to say the body of an "until" loop
is executed as long as the test condition evaluates to a false value, or,
phrased another way, until the test condition evaluates to a true value.
.Bd -literal -offset left
> (setq n 10) 
10 
> (while n 
>>> (print n) 
>>> (newline) 
>>> (setq n (- n 1)))
10 9 8 7 6 5 4 3 2 1 
1 
> (setq n 0)
0
> (setq l ())
()
> (until (eq n 10)
>> (setq l (cons (inc n) l)))
1
> l
(10 9 8 7 6 5 4 3 2 1)
.Ed
.Pp
The final 1 here is the return value of the "while" form.
.Pp
.Pp
.Pp
.Ss do: (do expr1 [expr2 ...])
The "do" intrinsic introduces a looping construct.  It accepts one or
more arguments.  All the arguments are evaluated in order.  If the last
expression evaluates to a true value (anthing other than 0, the empty string,
or the empty list), then all the argument expressions are evaluated again.
This process repeats until the last expression evaluates to a "false" value,
when the loop exits without further evaluation.  A do-loop will therefore
evaluate its argument expressions one or more times.  The "do" intrinsic
returns the value of the failed test expression.
.Bd -literal -offset left
> (set 'n 0) 
0 
> (do 
>> (print n) 
>> (newline)
>> (set 'n (+ n 1))) 
> (< n 10)) 
0 1 2 3 4 5 6 7 8 9 
1 
>
.Ed
.Pp
The final 1 here is the return value of the "do" form.
.Pp
.Pp
.Pp
.Ss fatal: (fatal) 
.Ss nofatal: (nofatal)
The "fatal" and "nofatal" intrinsics, accept no arguments, and determine if
errors encountered during evaluation will cause the interpreter to exit.  When
the interpreter starts it is in the "nofatal" state.  If an error is
encountered in the "nofatal" state, the stack will unwind back to the toplevel
interpreter prompt.  After "fatal" has been invoked, all errors which stop
evaluation will additionally cause the interpreter to exit.  Invoking "die"
when in the "fatal" state, will also cause the interpreter to exit up return
to toplevel.
.Pp
.Pp
.Pp
.Ss die: (die [expr...])
Intrinsic "die" forces the interpreter to stop evaluating any lisp it may
be evaluating and return to the toplevel.  The optional arguments are passed
to the "warn" intrinsic.  Invoking "die" when "fatal" has been invoked will
cause the interpreter to exit, upon return to toplevel.
.Pp
.Pp
.Pp
.Ss throw: (throw expr)
.Ss catch: (catch [expr1]...)
.Pp
The "catch" intrinsic together with the "throw" intrinsic provide a means
of performing non-local exits.  The arguments to catch are evaluated in an
implicit "progn" and the result of the evaluation of the last expression
is returned.  If, however, a "throw" form is encountered while evaluating
any of the arguments to "catch" then the thread of execution returns to
the catch form.  The "catch" form immediately returns, with the result of
evaluating the argument to the "throw" form becoming the return value of
the "catch" form.  A "throw" is caught by its enclosing "catch" form, if
there is one, or if there is no enclosing "catch" form, the interpreter
returns to the toplevel, just as if the "die" intrinsic has been invoked.
If the "fatal" intrinsic has been previously invoked, an uncaught "throw"
will cause the interpreter to exit.
.Bd -literal -offset left
> (catch 
>> (print (catch 0 (throw 'hello) 2 3 4)) 
>> (newline) 
>> (throw 'goodbye) 
>> (print 'not_reached)) 
hello 
goodbye
>
.Ed
.Pp
.Pp
.Pp
.Ss stringify: (stringify expr...)
The "stringify" intrinisc accepts one or more arguments which must all
evaluate to some sort of atom, and converts each atom's print syntax into a
string.  Then all the strings are concatenated into one string, which is
returned.
.Bd -literal -offset left
> (stringify 12 ","  43) 
"12,43"  
> (stringify 'hello " "  'there) 
"hello there"
.Ed
.Pp
.Pp
.Pp
.Ss digitize: (digitize expr)
Intrinsic "digitize" accepts one argument which must evaluate to an atom and
converts it to a number, if possible.  Attempting to apply digitize to a
non-number will cause it to return 0.
.Bd -literal -offset left
> (digitize '42) 
42 
> (digitize "-3") 
-3
.Ed
.Pp
.Pp
.Pp
.Ss mapcar: (mapcar expr1 expr2)
The "mapcar" library function accepts two arguments, the first of which
must evaluate to a monadic closure or a monadic intrinsic function or an
intrinsic which may accept one or more arguments, such as the "print"
intrinsic, while the second argument must evaluate to a list.  The macro
applies the function to each element of the list, returning a new list
containing the results of each evaluation.
.Bd -literal -offset left
> (set 'l '(a b c)) 
(a b c) 
> (set 'f (lambda (n) (list n n n))) 
<CLOSURE#23>
> (mapcar f l) 
((a a a) (b b b) (c c c))
.Ed
.Pp
.Pp
.Pp
.Ss map: (map expr1 expr2 ...)
The "map" library function is a generalized version of "mapcar".  The
function accepts two or more arguments, the first of which must evaluate to
a closure or an intrinsic function, while the remaining arguments must all
evaluate to lists of the same length.  The function passed as the first
argument must accept as many arguments as there are list arguments.  A new
list is returned consisting of the result of successively applying the
function to a grouping of elements from each list.  The elements of the
lists are successively grouped according to position.
.Bd -literal -offset left
> (map (lambda (x y z) (+ x y z)) '(1 2 3) '(4 5 6) '(7 8 9))
(12 15 18)
.Ed
.Pp
.Pp
.Pp
.Ss remove: (remove expr1 expr2)
Library function "remove" accepts two arguments, an expression of any type,
and a list, respectively, and returns a new list which is a copy of the
argument list, with all occurrences of the first argument removed.  The
function uses library function "equal" to test for equivalency.
.Bd -literal -offset left
> (remove 'a '(a a a b)) 
(b)
.Pp
> (remove '(a b) '((a b) c (a b) (a b) d e f g)) 
(c d e f g)
.Ed
.Pp
.Pp
.Pp
.Ss nthcdr: (nthcdr expr1 expr2)
Library function "nthcdr" accepts two arguments, the first of which with
must evaluate to a list, and the second of which must evalute to a positive
integer, N.  The cdr of the list is taken N times and returned.
.Bd -literal -offset left
> (nthcdr '(a b c) 1)
(b c)
> (nthcdr '(a b c) 2)
(c)
> (nthcdr '(a b c) 3)
()
> (nthcdr '(a b c) 4) ; Error, stops evaluation.
<CDR>: argument is empty list.
<SET>: evaluation of argument 2 failed.
<WHILE>: error evaluating body expression 1.
<WHILE>: evaluation of body failed.
nthcdr: error evaluating body expression 1.
.Ed
.Pp
.Pp
.Pp
.Ss nth: (nth expr1 expr2)
The "nth" library function passes is arguments to "nthcdr" and if the value
returned from that function is not the empty list, "nthcar" returns the car
of that list, otherwise it returns the empty list.  This behaviour can be
thought of as returning the "nth" element of a list, with the elements
numbered starting from zero.  Implied in this behaviour is the fact that
invoking "nth" with an index value equal to the length of the list will
return the empty list.  Invoking "nth" with values greater than this will
generate an error which will stop evaluation.
.Bd -literal -offset left
> (set 'l '(a b c))
(a b c)
> (let ((a 0)
        (len (length l)))
   (while (<= a len)
>>    (print (nth l a))
>>    (newline))
a
b
c
()
0 ; This is the return value of "while"
> (nth '(a b c) 4)
<CDR>: argument is empty list.
<SET>: evaluation of argument 2 failed.
<WHILE>: error evaluating body expression 1.
<WHILE>: evaluation of body failed.
nthcdr: error evaluating body expression 1.
lambda: evaluation of argument 1 failed.
nth: error evaluating body expression 1.
.Ed
.Pp
.Pp
.Pp
.Ss member: (member expr1 expr2)
Library function "member" accepts two arguments, the first of which can 
evaluate to any lisp object, while the second must evaluate to a list.
If the first argument is "equal" to any element of the list 1 is returned,
else 0 is returned.
.Bd -literal -offset left
> (member 'a '(a b c))
1
.Ed
.Pp
.Pp
.Pp
.Ss reverse: (reverse expr1)
Library function "reverse" accepts one argument, which must evaluate to a
list or a string.  If a list the function creates a new list containing the
same elements as the original list, in reverse order.  If the argument is a
string the function creates a new string consisting of the same characters
as the original, but in reverse order.
.Bd -literal -offset left
> (reverse '(a b c)) 
(c b a) 
> (reverse "hello") 
"olleh"
.Ed
.Pp
.Pp
.Pp
.Ss append: (append expr1 expr2)
The "append" intrinsic accepts two arguments which both must
evaluate to lists, and creates a new list consisting of the elements of the
first list followed by the elements of the second.
.Pp
.Pp
.Pp
.Ss loop: (loop expr1 ...)
The "loop" intrinsic implements an infinite loop.  The function accepts one
or more arguments, and evaluates them in order, repeatedly.  The return
value of each body expression is discarded.  The only way to exit a "loop"
loop is to wrap it in a "catch" and invoke "throw" in the body:  
.Bd -literal -offset left
> (catch 
>> (loop
>>> (if (setq line (getline))
>>>> (print line)
>>>> (throw line))))
.Ed
.Pp
.Pp
.Pp
.Ss iterate: (iterate expr1 ...)
The "iterate" intrinsic executes expressions a specified number of times.
Unlike the "for" intrinsic it does not make an index variable visible to
the expressions in its body.  The function accepts one or more arguments,
the first of which must evaluate to a number and specifies the number of
times the loop will execute.  A negative repeat count will be converted
into a positive value by converting it to its absolute value.  The
subsequent expressions, if any, are optional.  If present, each subsequent
expression will be executed in order on each iteration of the loop.  The
result of evaluating the last body form in the loop, on the last iteration,
will be returned, or if no further arguments were provided beyond the
repeat count, the repeat count will be returned.  The "iterate" intrinsic
is the fastest means of iteration available in the Munger interpreter.
.Bd -literal -offset left
> (iterate 10 (print "A"))
AAAAAAAAAA1
.Ed
.Pp
The 1 is the return value of the "print" intrinsic, which is in turn the
return value of "iterate".
.Pp
.Pp
.Pp
.Ss for: (for expr1 expr2 ...)
The "for" intrinsic executes loops with an index variable set to each of a
range of specified fixnum values upon each iteration.  It is the most
efficient means of iterating a fixed number of times, if the loop body
needs a visible index variable.  Two or more arguments must be provided.
The first is expected to be a three element list consisting of a symbol to
be used for the loop index variable, an expression evaluating to the
initial number value for the index variable, and an expression evaluating
to the number value the index variable should have on the last iteration of
the loop.  If the start value is less than or equal to the stop value, the
index variable will be incremented by one on each iteration, otherwise it
will be decremented by one on each iteration.  Side effects on the index
variable are allowed in the body of the loop, so, for example, if the user
wishes the index variable to be incremented by 2 instead of 1, he or she
may add an extra expression to increment the variable to the body of the
loop.  Each of the remaining arguments from the second argument on, will be
evaluated during each iteration of the loop.  The "for" function always
returns the value of the last body expression evaluated on the last
iteration.
.Pp
The binding for the symbol used for the index variable is semi-statically
scoped, which is to say both its visibility and its extent are limited to
the body of the "for" expression.  If a closure is formed in the body of a
"for" loop, it will close over the current lexical environment, including
the binding of the index variable, but this binding will be removed from
the lexical scope when the "for" finishes.
.Bd -literal -offset left
> (set 'a 5)
5
> (for (a a 1)
>> (for (a a (+ a a))
>>> (print a " "))
>> (newline))
5 6 7 8 9 10
4 5 6 7 8
3 4 5 6 
2 3 4
1 2
1 ; return value of inner "newline"
>
.Ed
.Pp
.Pp
.Pp
.Ss downcase: (downcase expr1 expr2)
.Pp
.Ss upcase: (upcase expr1 expr2)
.Pp
The "upcase" library function accepts two arguments, the first of which
must evaluate to a string, and the second of which is treated as a logical
true or false value after evaluation.  If the second argument evaluates to
a true value, then "upcase" will return a new string consisting of all the
characters of the first argument string, but with any lowercase alphabetic
characters replaced with their corresponding uppercase characters.  If the
second argument evaluates to a false value, then only the first lowercase
alphabetic character encountered in the first argument will be so
converted.  In either case, if no lowercase alphabetic characters are
present in the first argument, the returned string will be an exact copy
of the first argument.
.Pp
The "downcase" library function behaves similarly to "upcase", except it
converts lowercase alphabetic characters to uppercase alphabetic
characters.
.Pp
.Pp
.Pp
.Ss alist_lookup: (alist_lookup expr1 expr2)
The "alist_lookup" library function accepts two arguments.  The second must
evaluate to an alist, while the result of evaluating the first argument is
treated as a key of the alist.  The function returns the cdr of the first
sublist of the alist whose car is equal to the key.
.Bd -literal -offset left
> (set 'y '((a b) (b c))) 
((a b) (b c)) 
> (alist_lookup 'b y) 
(c) 
>
.Ed
.Pp
.Pp
.Pp
.Ss alist_remove: (alist_remove expr1 expr2)
The "alist_remove" library function accepts two arguments.  The second must
evaluate to an alist, while the result of evaluating the first argument is
treated as a key of the alist.  The function returns a copy of the alist
with the first sublist whose car is the key, removed.
.Bd -literal -offset left
> (set 'y '((a b) (b c))) 
((a b) (b c)) 
> (alist_remove 'b y) 
((a b)) 
> y 
((a b) (b c)) 
>
.Ed
.Pp
.Pp
.Pp
.Ss alist_replace: (alist_replace expr1 expr2 expr3)
The "alist_replace" library function accepts three arguments.  The second
must evaluate to an alist, while the result of evaluating the first is
treated as a key of the alist.  The third argument must evaluate to a list.
The function returns a copy of the alist with the first sublist whose car
is the key, removed, if such a sublist exists, and containing a new sublist
consisting of the key and the elements of the third list.  The new sublist
is added to the front of the alist copy.
.Bd -literal -offset left
> (set 'y '((a b) (b c))) 
((a b) (b c)) 
> (alist_replace 'b y '(d e f)) 
((b d e f) (a b))
.Pp
> (set 'y ()) 
() 
> (alist_replace 'b y '(d e f)) 
((b d e f)) 
>
.Ed
.Pp
.Pp
.Pp
.Ss when: (when expr1 expr2...)
The "when" intrinsic is a conditional.  It accepts one or more arguments, the
first being the test condition.  If the test condition evaluates to a true
value (anything but 0, the empty string, or the empty list), then the
succeeding arguments are evaluated in order, and the result of evaluating the
last argument is returned.  Otherwise, if the test condition evaluated to a
false value, that value is returned.
.Bd -literal -offset left
> (when (eq 1 1) 
>> (print 'hello) 
>> (newline)) 
hello 
1 
>
.Ed
.Pp
.Pp
.Pp
.Ss unless: (unless expr1 expr2...)
The "unless" intrinsic is a conditional, which operates logically opposite to
the "when" intrinsic.  The function accepts one or more arguments, the first
of which is the test condition.  If the first argument evaluates to a false
value (0, the empty list, or the empty string), the succeeding arguments are
evaluated in order, and the result of evaluating the last argument is
returned.  Otherwise, if the first argument evaluated to a true value, that
value is returned.
.Bd -literal -offset left
> (unless (eq 1 1) 
>> (print 'hello) 
>> (newline)) 
1 
>
.Ed
.Pp
.Pp
.Pp
.Ss apply: (apply expr1 expr2)
The "apply" macro accepts two arguments, the first of which must evaluate
to a function and the second of which must evaluate to a list.  the
function is called with all the elements of the second argument as
arguments.  All "apply" does is cons its first element onto its second and
evaluate the result.
.Bd -literal -offset left
> (apply 'set '('q 2)) 
2 
.Ed
.Pp
.Pp
.Pp
.Ss inc: (inc symbol [expr])
.Ss dec: (dec symbol [expr])
The "inc" and "dec" intrinsics increment and decrement a numberical value
bound to a symbol, and rebind the symbol to the resultant value.  Both
functions accept either one or two arguments.  The first argument is not
evaluated, and must be a symbol currently bound to a number.  The second
optional argument, if present, must evaluate to a number.  The value bound
to the symbol is incremented or decremented by the value specified by the
second argument, or if the function were not presented with a second
argument, by 1, and this new value is then bound to the symbol.  The new
value is returned.
.Bd -literal -offset left
> (set 'a 1) 
1 
> (inc a) 
2 
> (dec a) 
1
> (inc a 10)
11
.Ed
.Pp
.Pp
.Pp
.Ss test: (test expr1)
The "test" intrinsic is used to test macro expansion.  Its lone argument
is NOT evaluated, and must be a list consisting of a macro application,
just as one would enter it at the interpreter prompt.  The function returns
the result of expanding the macro.
.Bd -literal -offset left
> (test (let ((f "foobar")) f)) 
((lambda (f) f) "foobar")
.Pp
> (test (setq a 2)) 
(set (quote a) 2)
.Ed
.Pp
.Pp
.Pp
.Ss continue: (continue)
The "continue" intrinsic, when invoked from inside a "while", "do", "for",
"iterate", or "loop" loop will cause the loop to skip any subsequent
expressions in the loop and jump to the top of the loop to continue with
the next iteration.  Note, that invoking "continue" inside of a do loop
will prevent the evaluation of the test condition.  An infinite loop may
result in certain situations.
.Pp
If "continue" is invoked outside of the body of a loop, then the stack
will unwind until the interpreter returns to the toplevel prompt.
.Bd -literal -offset left
> (set 'n 10) 
10 
> (while n 
>> (print n) 
>> (newline)
>> (set 'n (- n 1)) 
>> (continue) 
>> (set 'n 0))
10 
9 
8 
7 
6 
5 
4 
3 
2 
1 
0
>
The last 0 is the return value of the "while" intrinsic.
.Ed
.Pp
.Pp
.Pp
.Ss block: (block) 
.Ss unblock: (unblock)
The "block" and "unblock" intrinsics accept no arguments, and determine
whether SIGINT will kill the interpreter or not.  The interpreter starts out
with an implicit call to "unblock."  Invoking "block" makes SIGINT harmless.
.Pp
.Pp
.Pp
.Ss exists: (exists expr)
The "exists" intrinsic accepts one argument which must evaluate to a string,
and attempts to stat() a file with the name specified by the string.  If the
call to stat() succeeds "exists" returns a value in the range of 1 to 8,
otherwise, it returns 0.  Possible successful return values are:
.Pp
.Bd -literal -offset left
1 == regular file
2 == directory
3 == character device
4 == block device
5 == fifo
6 == symbolic link
7 == socket
8 == unknown type
.Ed
.Pp
.Pp
.Pp
.Ss stat: (stat expr)
The "stat" intrinsic accepts one argument, which must evaluate to a string.
The argument is interpreted as a filename to be passed to the stat() system
call.  The function returns a list.  If the specified file does not exist, the
returned list will be empty.  Any other error encountered will stop evaluation.
Upon successful return of stat(), a five-element list will be returned,
containing, in order, a string containing the user name associated with the
uid of the file, or if the uid does not map to a user on the system, the uid
itself expressed as an integer, a string containing the group name associated
with the gid of the file, or if the gid does not map to a group on the system,
the gid itself expressed as an integer, the time of the last access of the
file, the time of the last modification of the file, and the size of the file.
The time values are expressed in seconds elapsed since the UNIX Epoch
(00:00:00 January 1, 1970 UTC).
.Bd -literal -offset left
> (stat (current))
("root" "wheel" 1097210850 1097210836 417214)
.Ed 
.Pp
.Pp
.Pp
.Ss rename: (rename expr1 expr2)
The "rename" intrinsic is used to rename entries in the filesystem.  Its
semantics are the same as those of the system call of the same name, for
which is a wrapper.  The function accepts two arguments, both of which must
evaluate to strings.  The first specifies the current name of the filesystem
entity, and the second is the new name it will have if the function succeeds.
1 is returned upon success, or in the case of system call failure, a string
describing the error is returned.  All other errors will stop evaluation.
.Pp
.Pp
.Pp
.Ss current: (current)
The interpreter maintains a pointer to an element of the argument list it
was started with.  The "current" intrinsic accepts no arguments, and returns a
string representing the current argument the argument pointer points to.  At
startup "current" will return the name the interpreter was invoked by, which
is always the first argument passed to programs running under UNIX.  See the
"prev" "next" and "rewind" intrinsics for accessing the other command line
arguments.
.Pp
.Pp
.Pp
.Ss prev: (prev)
The "prev" intrinsic steps the argument pointer back to the previous command
line argument, if one exists.  If no previous argument exists, "prev" returns
zero, otherwise the previous argument is made the current argument and is
returned as a string.
.Pp
.Pp
.Pp
.Ss next: (next)
The "next" intrinsic steps the argument pointer forward to the next command
line argument, if one exists.  If no further arguments exists, "next" returns
zero, otherwise the next argument is made the current argument, and is
returned as a string.
.Pp
.Pp
.Pp
.Ss rewind: (rewind)
The "rewind" intrinsic accepts no arguments, and sets the interpreter's
command-line argument pointer to point to the first argument, and returns it
as a string.  The first argument is always the name by which the interpreter
program was invoked.
.Pp
.Pp
.Pp
.Ss interact: (interact)
The "interact" intrinsic causes the interpreter to interrupt running the
current program and work interactively with the user.  It is intended for use
in programs which need to make the interpreter toplevel temporarily available
to the user.  The function accepts no arguments, and causes the interpreter to
enter a new read-eval-print loop.  To exit the loop and allow the program
which invoked "interact" to continue, the Control-D may be inputted by itself
on a line, or the symbol "_" may be input at toplevel.  The function always
returns 1.  Recursive invocations of this function are not permitted, and will
stop evalaution.
.Pp
.Pp
.Pp
.Ss pwd: (pwd)
The "pwd" intrinsic accepts no arguments, and returns the current working
directory of the interpreter as a string.
.Pp
.Pp
.Pp
.Ss let: (let [list] expr2 ...)
The "let" intrinsic introduces new local lexical bindings and evaluates a list
of expressions with those new bindings in place.  The bindings are removed
when the "let" form returns.  The first argument is not evaluated and must be
a parameter list of the form: ((symbol1 expr1) (symbol2 expr2)...).  The
result of evaluating each exprN in the current scope gets bound to its paired
symbolN in the new scope.  The remaining arguments, if any, are evaluated in
the new scope.  The result of the last evaluation performed is returned.
.Bd -literal -offset left
> (let ((a 1) (b 2)) 
>> (print "a: "  a "  b: "  b) 
>> (newline))
a: 1
b: 2 
1 ; This is the return value of (newline)
> (set 'a 2) 
2 
> (let ((a (* a a))) 
>> (while a 
>>>> (print "a: "  a)
>>>> (newline) 
>>>> (set 'a (- a 1)))) 
a: 4 
a: 3 
a: 2 
a: 1 
0
> a 
2
.Ed
.Pp
.Pp
.Pp
.Ss letn: (letn [list] expr2 ...)
Intrinsic "letn" introduces new local lexical bindings and evaluates a list of
expressions with those new bindings in place.  The bindings are removed when
the "letn" form returns.  The first argument is not evaluated and must be a
parameter list of the form: ((symbol1 expr1) (symbol2 expr2)...).  The result
of evaluating each exprN in the current scope gets bound to its paired symbolN
in the new scope.  The remaining arguments are evaluated in the new scope.
The value of the last evaluation performed is returned.  Intrinsic "letn" is
different from intrinsic "let" in that each binding made to symbolN is visible
in exprN+1, exprN+2, etc.  This form is called "let*" in Scheme and Common
Lisp.
.Bd -literal -offset left
> (letn ((a 1)
>>>      (b (+ a 1)))
>> (print "b: "  b)
>> (newline))
b: 2
1
>
.Ed
.Pp
.Pp
.Pp
.Ss letf: (letf [list] expr2 ...)
Macro "letf" is the equivalent to a named let in Scheme.  It provides a means
of creating and applying a temporary function capable of recursing on its
name.  The macro accepts two or more arguments, the first of which must be a
symbol, while the second must be a list of the form: ((symbol1 expr1) (symbol2
expr2) ...).  The result of evaluating each exprN in the current scope gets
bound to its paired symbolN in a new local lexical environment.  The symbol
passed as the first argument is visible in the new environment to enable
recursion by name.  The remaining arguments are then evaluated in the new
environment, with the value of the last evaluation performed being returned.
.Bd -literal -offset left
> (setq a 10)
10
> (letf fact ((n a))
>> (if (< n 2)
>>>    n
>>>    (* n (fact (- n 1)))))
3628800
>
.Ed
.Pp
.Pp
.Pp
.Ss tailcall: (tailcall expr1 ...)
The "tailcall" intrinsic accepts one or more arguments, the first of which
must evaluate to a closure or 0, while the remaining arguments, if any, are
the arguments to the closure being tail-called, and must be of the correct
number and type for the particular closure being invoked.  A first argument
of 0 indicates the tail-call is a directly recursive call of the
currently-executing function.  This allows anonymous functions to call
themselves tail-recursively.  Using tail-calls to implement iteration is
inefficient in Munger, and should be avoided.  See the entries for the
"for" and "iterate" intrinsics for how to iterate efficiently.
.Pp
Tail-calls are used to prevent the control stack from growing unnecessarily
during recursive function calls.  Use "tailcall" whenever you are invoking
a function from a tail position in another function.
.Pp
Some tail-recursive functions do all their computation during the "descent"
stage of recursion, and only return the final computed value during the
"ascent" stage when they "unwind".  This means the recursive call is in
tail position because nothing happens after it has returned.  Each
invocation simply returns to its caller.  Invoking the recursive calls with
"tailcall" will cause each recursive invocation to replace the current
context on the call stack with its own context, so that when the final
recursive invocation returns, it returns to the caller of the first
invocation.  No "unwinding" occurs.  Each invocation of the function
therefore has a constant continuation, and the whole computation is
effectively turned into a loop, inhibiting potentially explosive growth of
the control stack.
.Pp
Other tail-recursive functions pass closures to each recursive invocation
of themselves to capture state as they descend, with each new closure
closing over the binding for the previous one.  When these functions
bottom-out, the current closure is invoked as the continuation of the
function.  It, in turn, performs part of a computation, then invokes the
continuation from the previous invocation, in tail position, to continue
the computation, and so on, until the primordial continuation given to the
toplevel invocation receives the final value of the computation.  Since
all invocations of the recursive function, and all invocations of the
dynamically-constructed continuations are in tail position, there is no
need to save their contexts on the call stack.  Every one can be invoked
with "tailcall", so that the primordial continuation simply returns to the
original caller of the first invocation of the recursive function.
Note that in this case, the savings in growth of the call stack, is lost
due to the growth of the heap from the creation of a new continuation at
each invocation.
.Pp
Note that invoking "tailcall" from a non-tail position, results in that
position becoming a tail position.  Any pending computation in the function
is abandoned.
.Pp
.Pp
.Pp
.Ss labels: (labels [list] expr2 ...)
The "labels" intrinsic temporarily binds a set of functions to a set of
symbols, such that each binding is visible in each function, allowing
recursive and mutually-recursive local functions to be defined in a new
environment.  The function accepts one or more arguments, the first of
which must be a list of lists where each sublist consists of a symbol
paired with a lambda or macro expression.  The remaining arguments are
evaluated in the new environment, with the value of the last evaluation
being returned.  An example will make this clear:  
.Bd -literal -offset left
> (labels ((even (lambda (n) (or (eq n 0) (tailcall odd (- n 1)))))
>>>        (odd  (lambda (n) (and (not (eq n 0)) (tailcall even (- n 1))))))
>> (print (even 11))
>> (newline))
0
1 ; This is the return value of the "newline".
.Ed
.Pp
Note that it may not be necessary to use "labels" to extend the current
lexical environment, depending on whether or not the programmer wishes to
limit the visibility of the functions created to the body of the "labels"
form, and whether or not there are any closures formed in the body which
need to see the new bindings after the "labels" has returned.  If both of
those behaviors are simultaneously needed, or if the interpreter is at
toplevel, "labels" must be used.  Otherwise, the current lexical
environment can be extended with the "extend" intrinsic.  One can achieve
the same effect as the above example inside a function body with:  
.Bd -literal -offset left
> (defun f ()
>> (extend 'even (lambda (n) (or (eq n 0) (tailcall odd (- n 1)))))
>> (extend 'odd (lambda (n) (and (not (eq n 0)) (tailcall even (- n 1)))))
>> (print (even 11))
>> (newline))
> (f)
0
1
.Ed
.Pp
The bindings created by "extend" have unlimited extent, so if closures were
formed in the body of "f" they would close over the new bindings, even if
the closures were closed before the invocations of "extend".  See the
entries for the "extend" and "dynamic_extent" intrinsics for more details
on extending the current lexical environment.
.Pp
.Pp
.Pp
.Ss extract: (extract expr)
The "extract" intrinsic extracts the lambda-expression from a function
closure.  It accepts one argument which must evaluate to a closure, and
returns the closure's lambda-expression.
.Bd -literal -offset left
> mapcar
<CLOSURE#17>
> (extract mapcar)
(lambda (f l) (if (pair l) (cons (f (car l)) (mapcar f (cdr l))) ()))
.Ed
.Pp
.Pp
.Pp
.Ss cond: (cond [list] ...)
Intrinsic "cond" is a multiple choice conditional.  The function accepts one
or more lists as arguments, which must consist of at least two elements, and
attempts to evaluate the first element of each argument list, in order, until
an evaluation returns a true value (anything but zero, the empty string, or
the empty list), when it attempts to evaluate the remaining elements of the
corresponding sublist.  The value of the last evaluation performed in the
sublist is returned.  If none of the first elements of any of the argument
lists evaluate to a true value, then the false value returned by the first
element of the last argument list will be returned.  By placing a first
element of 1 in the final sublist, a catch-all else clause may be created.
.Bd -literal -offset left
> (set 'a 10) 
10 
> (cond ((eq a 12) 'no) 
>>      ((eq a 10) 'yes))
yes 
>
.Ed
.Pp
.Pp
.Pp
.Ss foreach: (foreach expr1 expr2)
The "foreach" library function applies a monadic function to every item in
a list.  Unlike "mapcar", this function does not return a list of the
return values.  The results of each function application are discarded.
"foreach" always returns the empty list.
.Bd -literal -offset left
> (set 'print_each 
>> (lambda (x) (print x) (newline))) 
<CLOSURE#34>
> (foreach print_each '(a b c d)) 
a
b
c
d 
()
>
.Ed
.Pp
.Pp
.Pp
.Ss +: (+ expr...)
Intrinsic "+"  accepts any number of arguments, evaluates them, and if all
have evaluated to numbers, adds the numbers together and returns the total.
.Bd -literal -offset left
> (+ 1 1 1 1) 
4 
> (+ 1 "hello") 
+: argument 2 did not evaluate to a number.
.Ed
.Pp
.Pp
.Pp
.Ss -: (- expr1 expr2)
Intrinsic "-"  accepts two arguments, both of which must evaluate to numbers,
and subtracts the second value from the first and returns the result.
.Bd -literal -offset left
> (- 1 2) 
-1 
> (- (+ 1 1) 2) 
0
.Ed
.Pp
.Pp
.Pp
.Ss *: (* expr...)
Intrinsic "*"  accepts one or more arguments which must all evaluate to
numbers, and multiplies all the values together and returns the result.
.Bd -literal -offset left
> (* 1 12 2) 
24 
> (* (+ 23 3) 4) 
104
.Ed
.Pp
.Pp
.Pp
.Ss /: (/ expr1 expr2)
Intrinsic "/" accepts two arguments both of which must evaluate to numbers,
and divides the second value into the first and returns the integer part of
the quotient.
.Bd -literal -offset left
> (/ 4 3) 
1
.Ed
.Pp
.Pp
.Pp
.Ss %: (% expr1 expr2)
Intrinsic "%" accepts two arguments, both of which must evaluate to numbers,
and divides the second value into the first, and returns the the remainder.
.Bd -literal -offset left
> (% 4 3) 
1
.Ed
.Pp
.Pp
.Pp
.Ss >: (> expr1 expr2)
Intrinsic ">" accepts two arguments, which both must evaluate to numbers,  and
returns 1 if the first value is larger than the second, or 0 otherwise.
.Bd -literal -offset left
> (> 4 3) 
1 
> (> -4 3) 
0
.Ed
.Pp
.Pp
.Pp
.Ss >=: (>= expr1 expr2)
Intrinsic ">=" accepts two arguments which both must evaluate to numbers, and
returns 1 if the first value is greater than or equal to the second value, or
0 otherwise.
.Bd -literal -offset left
> (>= 4 3) 
1
.Ed
.Pp
.Pp
.Pp
.Ss <: (< expr1 expr2)
Intrinsic "<" accepts two arguments, both of which must evaluate to numbers,
and returns 1 if the first value is less than the second, or 0 otherwise.
.Bd -literal -offset left
> (< 4 3) 
0
.Ed
.Pp
.Pp
.Pp
.Ss <=: (<= expr1 expr2)
Intrinsic "<=" accepts two arguments, both of which must evaluate to numbers,
and returns 1 if the first value is less than or equal to the second, or 0
otherwise.
.Bd -literal -offset left
> (<= 3 4) 
0
.Ed
.Pp
.Pp
.Pp
.Ss abs: (abs expr)
Intrinsic "abs" evaluates its lone argument, and if it evaluates to a number,
returns the absolute value of the number.  Passing a non-number to "abs" will
stop evaluation with an error.
.Pp
.Pp
.Pp
.Ss intern: (intern expr1)
The "intern" intrinsic accepts one argument, which must evaluate to a
string, and converts it into a symbol token consisting of the same
sequence of characters, without the enclosing quotation marks.  Passing
a non-string to "intern" will generate an error which stops evaluation.
.Bd -literal -offset left
> (set 'hello 4) 
4 (eval (intern "hello")) 
4
.Ed
.Pp
.Pp
.Pp
.Ss char: (char expr1)
The "char" intrinsic accepts one argument, which must evaluate to a number
between 1 and 127, and creates a one-character string consisting of the
character corresponding to the character code specified by that number.
.Pp
.Pp
.Pp
.Ss code: (code expr1)
The "code" intrinsic accepts one argument which must evaluate to a string
and returns a number representing the character code of the first character of
the string.
.Pp
.Pp
.Pp
.Ss open: (open)
The "open" intrinsic creates a new text buffer and makes the new buffer the
active buffer.  The buffer number of the new text buffer is returned on
success.  Errors stop evaluation.  Buffer numbers are whole numbers, and can
be used by the "switch" intrinsic to change the active buffer.
.Pp
.Pp
.Pp
.Ss close: (close)
The "close" intrinsic closes the active text buffer, and makes the
most-recently opened buffer that has not been closed the active buffer.  1
is returned on success.  Errors stop evaluation.
.Pp
.Pp
.Pp
.Ss insert: (insert expr1 expr2 expr3)
The "insert" intrinsic inserts a line into the active buffer.  It accepts
three arguments.  The first argument must evaluate to a positive number and
represents an index position in the buffer.  The second argument, which must
evaluate to a string, is the data to be inserted.  The third argument, which
must evaluate to a number, specifies whether the data should be inserted
before the specified index position, inserted after the specified index
position, or should overwrite the contents of the specified index position.  A
value of 0 indicates the index position should be overwritten.  A positive
value indicates the data should be inserted after the specified index.  A
negative value indicates the data should be inserted before the specified
index.  If the specified index position does not exist it will be created, and
all the index positions preceding it in the buffer will also be created and
initialized to be empty.  1 is returned on success, 0 on failure.
.Bd -literal -offset left
> (open) 
1 
> (insert 1 "This is the first line." 0)
1
.Ed
.Pp
.Pp
.Pp
.Ss delete: (remove expr1)
The "delete" intrinsic removes a line from the active buffer.  The function
accepts one argument which must evaluate to a positive integer specifying
the index of the line in the buffer to be deleted.  1 is returned on success,
0 on failure.
.Pp
.Pp
.Pp
.Ss retrieve: (retrieve expr1)
The "retrieve" intrinsic returns the contents of a specified index position in
the active buffer, as a string.  The function accepts one argument which must
evaluate to a positive integer representing the index of the desired line.  An
error is generated if the specified index does not exist.
.Pp
.Pp
.Pp
.Ss lastline: (lastline)
The "lastline" intrinsic accepts no arguments, and returns the index value of
the last line in the active buffer.  Any error encountered will stop the
interpreter.
.Pp
.Pp
.Pp
.Ss filter: (filter expr1 expr2 expr3)
The "filter" intrinsic sends a range of lines from the active buffer to an
external filter program, and replaces the lines in the buffer with the output
from the filter program.  The function accepts three arguments.  The first two
arguments must evaluate to numbers and inclusively specify the range of lines
to be sent to the filter program.  The first index does not have to be less
than the second index.  The lines, however are always processed in ascending
order, regardless of how the range was specified.  An error is generated if
any of the lines in the specified range do not exist.  The third argument must
evaluate to a string, representing the command line used to launch the filter.
It is passed to /bin/sh for interpretation, and so may contain any expression
that program understands.
.Pp
Upon success, "filter" returns the number of lines read from the stdout of
the inferior process.  If the inferior shell cannot be run or if it
encounters an error which causes it to exit with a non-zero exit status, or
if it is terminated by a signal, or if the specified program cannot be run
by the inferior shell, or if the specified program does not produce any
output, "filter" will return 0 and the lines in the buffer will remain
unaltered.
.Pp
.Pp
.Pp
.Ss write: (write expr1 expr2 expr3 expr4 [expr5])
The "write" intrinsic writes content from the active buffer to a file.  The
function accepts four or five arguments.  The first two arguments must
evaluate to positive integers, and specify a range of lines, inclusively, to
be written out.  The third argument must evaluate to a string representing the
filename to be written to.  The fourth argument must evaluate to a number, and
indicates whether the interpreter should attempt to get an exclusive lock on
the file before writing to it.  A non-zero value means to lock the file, while
a value of 0 means to not lock the file.  The optional fifth argument, if
present, must evaluate to a number, and specifies whether the lines from the
buffer should be appended to an already-existing file, or if a new file should
be created, overwriting any existing file of the same name.  A non-zero value
means to append, while a value of 0 results in a new file being created.  If
the fifth argument is not present, an implicit fifth argument of 0 is assumed.
Empty files can be created by passing 0 for both the first and second
arguments.  "write" creates files with both read and write permissions enabled
for the owner, and read permission enabled for everyone else (mode 644).
.Pp
The first index argument does not have to be less than the second index
argument, but the lines in the region so specified will be written to the
file in ascending order, regardless of how the region was specified.
.Pp
Whitespace in the filename can either be present literally, or represented
with the \\t and \\b escapes to represent the tab and the space character,
respectively.  This means if you wish to use a literal \\b or \\t in the
filename, you must escape the backslash itself and use \\\\\\\\b or
\\\\\\\\t instead.  Remember to embed a single backslash into a string we
must escape it with another backslash, so to embed two backslashes, we need
to escape each of them with another backslash, totalling four backslashes. 
If this confuses you, try rereading the section on strings at the top of
this document.
.Pp
The number of lines written is returned on success.  If an error is
encountered opening the file, a string describing the error is returned.
All other errors stop evaluation.
.Pp
.Pp
.Pp
.Ss read: (read expr1 expr2)
The "read" intrinsic inserts all the lines from a file into the active
buffer, inserting the lines after a specified line.  The function accepts two
arguments.  The first argument must evaluate to a number representing the
index value of the line after which to insert the newly-read lines.  If the
buffer is empty, the first argument must be 0, or an error is generated.
Similarly, to insert lines at the beginning of the buffer, the first argument
should be 0.  The function returns the number of lines read on success, -1 if
the file to be read doesn't exist, and -2 if permission to access the file is
denied.  Any other failure of the read() system call will return a string
describing the error.  Other errors will stop evaluation.
.Pp
Whitespace in the filename can either be present literally, or represented
with the \\t and \\b escapes to represent the tab and the space character,
respectively.  This means if you wish to use a literal \\b or \\t in the
filename, you must escape the backslash itself and use \\\\\\\\b or
\\\\\\\\t instead.  Remember to embed a single backslash into a string we
must escape it with another backslash, so to embed two backslashes, we need
to escape each of them with another backslash, totalling four backslashes. 
If this confuses you, try rereading the section on strings at the top of
this document.
.Pp
.Pp
.Pp
.Ss empty: (empty)
The "empty" intrinsic accepts no arguments, and removes all data from the
active buffer.  All data in the active buffer are permanently lost.
.Pp
.Pp
.Pp
.Ss slice: (slice expr1 expr2 expr3 expr4 expr5)
The "slice" intrinsic returns part of a line in the active buffer or a
description of part of a line in the buffer.  The function accepts five
arguments, which must all evaluate to numbers.  The first argument specifies
the index value of the line to be sliced in the buffer.  The second argument
specifies the character index where the slice starts.  The third argument
specifies the length of the slice in characters.  If the length argument is
zero, this is interpreted as meaning, "to the end of the line."  The fourth
argument specifies where tabstops occur.  Tabs are expanded before the slice
is taken, so argument 2 refers to the "screen" x-coordinate, which may be
different from the actual character located at that index position in the
buffer, due to tab expansion.  For example, a fourth argument of 3 indicates
that tabstops are considered to occur every three columns.
.Pp
The fifth argument specifies what the function returns.  A value of 0
indicates the caller wishes to receive the specified slice as a string.  A
value of 1 indicates the caller wishes to receive a two-element list
describing the specified slice.  The first element of this list is the
length of the slice, in characters, which may be less than the specified
length, if the specified length extended past the end of the line.  This is
the actual length of the slice in the buffer before tab expansion.  The
second element is the number of extra characters which would be added to
the line during tab expansion from the beginning of the line to the end of
the specified slice.
.Pp
.Pp
.Pp
.Ss find: (find expr1 expr2 expr3 expr4 expr5)
The "find" intrinsic searches a specified range of lines in the active
buffer for a match on a specified regular expression.  The function accepts
five arguments.
.Pp
The first three arguments, and the fifth argument, must all evaluate to
numbers, while the fourth argument must evaluate to a compiled regular
exression object.  The first argument specifies the direction of the search:
a positive value causes the search to proceed forward in the buffer, while a
negative value causes the search to proceed backwards in the buffer.  The
second argument is interpreted as the line number at which to start the
search.  The third argument is interpreted as the character within the line at
which to start the search.  Remember that lines in the buffer are indexed from
1, while characters in lines are indexed from 0.  The fifth argument specifies
whether the search should "wraparound" if it fails.  A non-zero value enables
wraparound, while 0 disables wraparound.
.Pp
For example, if a search were in the forward direction and failed, a fifth
argument of 1 would cause the search to begin again at the beginning of the
buffer, looking for matches before the specified starting position.  The
fourth argument is the compiled regular expression object to search for
matches with.  A match starting exactly at the specified starting place of the
search will be ignored, and the position of the next non-overlapping match, if
any, will be sought.  Newlines are temporarily removed from the end of buffer
lines before a match is attempted.  This means ^$ will match empty lines.
.Pp
The function returns a list of three numbers.  If a match were found, the first
element will be the index of the line containing the match, while the second
element will be the character index in the line of the start of the text that
matched, and the third element will be the length, in characters, of the
match.  If no match occurred, all three values will be zero.
.Pp
The three returned values can be used to pluck out the text of a match
from the buffer:
.Bd -literal -offset left
> (open)
0
> (insert 1 "this is the first line in the buffer." 0)
1
> (set 'f (find 1 1 0 (regcomp "buffer") 0)
(1 30 7)
> (substring (retrieve (car f)) (cadr f) (car (cddr f)))
"buffer"
.Ed
.Pp
The following function will count the number of blank lines in the buffer,
assuming the buffer has been opened and loaded with text.  Newlines are
removed from the end of lines before the regular expression is applied, so
the regular expression will match blank lines.
.Bd -literal -offset left
(set 'blank
(lambda ()
.Pp
   (let ((idx 1)
         (regexp (regcomp "^[\\b\\t]*$")) 
         (count 0))
.Pp
       (while (set 'idx (car (find 1 idx 0 regexp 0)))
         (inc count))
.Pp
       count)))
.Ed
.Pp
.Pp
.Pp
.Ss buffer: (buffer)
The "buffer" intrinsic accepts no arguments, and returns the buffer number of
the active text buffer, or -1 if no buffers have been opened.
.Pp
.Pp
.Pp
.Ss buffers: (buffers)
The "buffers" intrinsic accepts no arguments, and returns a list of whole
numbers representing the buffer numbers of all currently open buffers.  If no
buffers have been open, the function returns the empty list.
.Pp
.Pp
.Pp
.Ss switch: (switch expr)
The "switch" intrinsic makes a specified buffer the active buffer.  The
function accepts one argument which must evaluate to the buffer number of
an open buffer, and makes that buffer the active buffer.  Errors stop
evaluation.
.Pp
.Pp
.Pp
.Ss setmark: (setmark expr1 expr2)
The "setmark" intrinsic is used to mark a line in the buffer for later
reference.  The function accepts two arguments, the first of which must evaluate
to an atom of any type, which names the bookmark.  The second argument must
evaluate to a valid line number of a line in the active buffer.  Upon success 1
is returned.  The mark will be adjusted accordingly after buffer insertions and
deletions in order to track the marked line.  Invoking "getmark" on a subsequent
occasion will return the marked line's current index position (line number).
.Pp
Bookmarks are local to the active buffer, and each buffer may have an unlimited
number of bookmarked lines.  The active set of bookmarks is switched when the
"switch" intrinsic changes the active buffer.  Only the bookmarks for the
active buffer can be altered or examined.
.Pp
.Pp
.Pp
.Ss getmark: (getmark expr1)
The "getmark" intrinsic is used to retrieve the line number of a marked line in
the active buffer.  The function accepts one argument which must evaluate to an
atom of any type, which names the desired bookmark.  The current index position
(line number) of the marked line will be returned if the line has not been
deleted from the buffer.  If the desired mark has not been set, "getmark"
returns 0.  If the marked line has been deleted, "getmark" returns -1.
.Pp
.Pp
.Pp
.Ss transfer: (transfer b1 f1 t1 b2 t2)
The "transfer" intrinsic copies a contiguous range of lines from one buffer
into another already-opened buffer.  The function accepts five arguments which
must all evaluate to numbers.  The first argument is the buffer number of the
source buffer.  The second and third arguments specify the starting and ending
lines, inclusive, of the range to be copied from the source buffer.  If the
second argument is greater than the third, then the lines will be copied in
reversed order into the destination buffer.  The fourth argument is the buffer
number of the destination buffer, while the fifth argument is the index in the
destination buffer, after which the copied lines will be inserted.  To copy
lines to the front of the destination buffer, a fifth value of 0 is used.
Whatever buffer was active when "transfer" was invoked will be active when
transfer returns.  The function returns 1 on success.  Any error will stop
evaluation.
.Pp
.Pp
.Pp
.Ss with_buffer: (with_buffer expr1 expr2 ...)
The "with_buffer" macro temporarily changes the active buffer.  The macro
accepts two or more arguments, the first of which must evaluate to the buffer
number of an open buffer.  This buffer is made the active buffer, and then the
arguments subsequent to the first are evaluated.  When the last argument has
been evaluated the buffer which was active previous to the invocation of
macro is made the active buffer once again.  The result of evaluating the
last argument is returned.
.Pp
.Pp
.Pp
.Ss version: (version)
The "version" intrinsic accepts no arguments and returns a list of two
numbers: the first being the major version number of the lisp interpreter, and
the second being the minor version number.
.Pp
.Pp
.Pp
.Ss gensym: (gensym)
The "gensym" intrinsic creates and returns a unique anonymous symbol, called a
gensym.  Gensyms cannot be named in your code, because the lisp reader does
not recognize the print syntax for a gensym.  Gensyms are useful when it is
necessary for a macro to create a working variable in its returned expression,
which must not conflict with other variables used in the program invoking the
macro.  The customary way to manipulate gensyms is to bind them to other
symbols, and evaluate these symbols in macro templates.
.Pp
The definition of the "with_buffer" macro from library.munger is presented
below as an example.
.Bd -literal -offset left
(let ((buff (gensym)))

   (set 'with_buffer
      (macro (x (y))

         (qquote
            (let ((,buff (buffer)))
               (switch ,x)
               (protect ,(cons 'progn y)
                  (switch ,buff)))))))
.Ed
.Pp
A gensym is bound to the symbol "buff" in a lexical closure surrounding the
macro definition.  This symbol's value, the gensym itself, is inserted into
the macro template at appropriate places to act as a temporary to store the
buffer current at the time the macro is invoked.  Because the gensym is
anonymous, we can be sure we are not shadowing a binding used by the
program in which the macro is invoked, which the code passed to the macro
as its second argument might modify.  Furthermore, every invocation of the
macro uses the same gensym, because it is created in the outer "let"
lexical closure enclosing the macro definition, but because we insert the
gensym into another "let" in the macro expansion, we can be sure that
nested invocations of "with_buffer" will only see their own lexical
binding.
.Pp
.Pp
.Pp
.Ss libdir: (libdir)
The "libdir" instrinic accepts no arguments and returns a string specifying
the location of the interpreter's library files as a fully-qualified directory
name, without the trailing virgule.
.Bd -literal -offset left
> (libdir) 
"/usr/local/share/munger"
>
.Ed
.Pp
.Pp
.Pp
.Ss strcmp: (strcmp expr1 expr2)
The "strcmp" intrinsic lexigraphically compares two strings.  The function
accepts two arguments which must evaluate to strings, and returns an
integer greater than, equal to, or lesser than zero, indicating that the first
string is either "greater than" (would sort after), "equal to" (identical), or
"less than" (would sort before) the second string.
.Bd -literal -offset left
> (strcmp "a" "b")
-1
> (strcmp "b" "a")
1
> (strcmp "a" "a")
0
> (strcmp "A" "a")
-32
.Ed
.Pp
.Pp
.Pp
.Ss substring: (substring expr1 expr2 expr3)
The "substring" intrinsic is provided for extracting substrings from strings,
using character indices.  The function accepts three arguments.  The first
must evaluate to a string, while the second and third must evaluate to whole
numbers.  The second argument specifies the character index (indices start at
0) where the desired substring begins.  The third argument is the number of
characters to include in the substring.  The "substring" intrinsic returns the
specified substring as a new string.  The second index may be zero, which is
interpreted to mean "to the end of the string."
.Bd -literal -offset left
> (substring "foobar" 0 0) 
"foobar" 
> (substring "foobar" 3 0) 
"bar" 
> (substring "foobar" 0 3) 
"foo" 
> (substring "foobar" 0 10) 
"foobar"
.Ed
.Pp
.Pp
.Pp
.Ss expand: (expand expr1 expr2)
The "expand" intrinsic performs tab expansion on arbitrary strings.  The
function accepts two arguments, the first of which must evaluate to a positive
integer, while the second must evaluate to a string.  The first value is
interpreted as the location of tabstops (ie., every expr1 characters).  A
new string is returned, which has the same content as the original second
argument, but with any tab characters expanded into an appropriate number
of spaces.  The number of spaces resulting from each expansion depends upon
the position of the specific tab character in its enclosing line.  An
expansion will always contain at least one space character, but may contain
up to expr1 space characters.  As well, the expansion of tabs ocurring
earlier in expr2 will influence the expansion of tabs ocurring later in
expr2.
.Pp
.Pp
.Pp
.Ss lines: (line)
The "lines" intrinsic returns the number of lines on the terminal device
on which the interpreter is running.  If the interpreter is not running on
a terminal device, the returned value cannot be predicted.
.Pp
.Pp
.Pp
.Ss cols: (cols)
The "cols" intrinsic returns the number of columns on the terminal device
on which the interpreter is running.  If the interpreter is not running on
a terminal device, then the returned value cannot be predicted.
.Pp
.Pp
.Pp
.Ss exit: (exit)
The "exit" interpreter accepts one argument, which must evaluate to a
number, and causes the interpreter to exit with that number as the exit
value returned to the system.
.Pp
.Pp
.Pp
.Ss complete: (complete expr )
The "complete" intrinsic performs filename completion.  The function
accepts one argument which must evaluate to a string and is the partial file
or directory name to be completed.  The function returns a list of one or more
strings.  The first element of the list is always the result of applying the
completion algorithm to the argument, and will be the same as the intial
argument if the completion algorithm could not add more characters to it.  If
more than one element is present in the list, more characters may have been
added to the initial argument, but it still did not unambiguously name a file.
The subsequent list elements will be preformatted lines of text containing all
the possible completions for the first element organized into a table the
width of the terminal device, and may be printed as is.
.Bd -literal -offset left
> (complete "/usr/share/pe") 
("/usr/share/perl/man/" "./     ../    man3/  whatis cat3/ 
                        ")
.Ed
.Pp
A leading ~ in the argument to "complete" will trigger home directory
abbreviation expansion, similarly to csh.  Home directory abbreviations are
expanded, but not completed.  For example, if the argument to "complete" is
either "~" or begins with "~/", these characters will be replaced with the
path to the current user's home directory, but arguments starting with text
matching this pattern: ~[^/]+ will have those characters replaced with the
path to the specified user's home directory, only if such a user exists.
The function will not attempt to complete an incomplete home directory
abbreviation.
.Pp
.Pp
.Pp
.Ss input: (input expr1 expr2)
The "input" intrinsic reads data from a process into the active buffer.  The
function accepts two arguments.  The first argument must evaluate to a whole
number and specifies the line number to insert the new data after, while
second argument must evaluate to a string specfying the command-line to pass
to the shell (/bin/sh) to launch the source process, and so may be any
expression that program understands.  The number of lines read is returned on
success.  If the inferior shell cannot find the specified program or the
process exits prematurely for any reason, "input" returns 0.  Any other errors
encountered will stop evaluation.
.Pp
Whitespace in the filename can either be present literally, or represented
with the \\t and \\b escapes the "substitute" and "match" intrinsics
recognize to represent the tab and the space character, respectively.
This means if you wish to use a literal \\b or \\t in the filename, you
must escape the backslash itself and use \\\\b or \\\\t instead.
.Pp
.Pp
.Pp
.Ss output: (output expr1 expr2 expr3)
The "output" intrinsic writes content from the active buffer to a process.
The fuction accepts three arguments.  The first two arguments must evaluate
to positive integers and specify the range of lines, inclusively, to be
written to the process.  The first index does not have to be less than the
second index, but the range of lines so specified will be written to the child
process in ascending order, regardless of how the range was specified.  The
third argument must evaluate to a string, and specifies the command-line to be
passed to the shell to launch the process, and so may be any expression that
program understands.  The number of lines written to the process is returned
upon success.  If the inferior shell cannot find the specified program or the
process exits prematurely for some other reason, "output" will return 0.  All
other errors will stop evaluation.
.Pp
Whitespace in the filename can either be present literally, or represented
with the \\t and \\b escapes the "substitute" and "match" intrinsics
recognize to represent the tab and the space character, respectively.
This means if you wish to use a literal \\b or \\t in the filename, you
must escape the backslash itself and use \\\\b or \\\\t instead.
.Pp
.Pp
.Pp
.Ss system: (system expr1)
The "system" intrinsic is an interface to the "system" system call.  The
function accepts one argument which must evaluate to a string, and passes
it to the shell for execution.  The function returns the exit status of the
shell; 127 if execution of the shell failed; -1 if fork() or waitpid() fails.
.Pp
.Pp
.Pp
.Ss maxidx: (maxidx)
The "maxidx" intrinsic accepts on arguments, and returns the highest possible
index in the buffer that the interpreter will recognize.
.Pp
.Pp
.Pp
.Ss chdir: (chdir expr1)
The "chdir" intrinsic changes the current directory.  The function accepts
one argument which must evaluate to a string, specifying the new current
working directory.  The function returns 1 on success or a string describing
the error condition on failure.
.Pp
.Pp
.Pp
.Ss table: (table)
The "table" intrinsic returns a new associative array table, which may be
used to store lisp objects indexed by atomic keys.  Table objects are
constants which evaluate to themselves.  See the "hash", "unhash", "keys",
and "values" intrinsics for the details on using tables.
.Pp
.Pp
.Pp
.Ss hash: (hash expr1 expr2 expr3)
The "hash" intrinsic is used to store data into a table.  The program accepts
three arguments, the first of which must evaluate to the table to be modified.
The second argument must evaluate to any atom, while the third argument can
evaluate to an object of any type.  The second argument becomes the "key"
associated with the third argument "value."  The "hash" intrinsic always
returns the result of evaluating the third argument.  Note that "lookup"
returns the empty list if a key has no association, so there is no way to
differentiate between a key associated with the empty list and a key with
no association.
.Bd -literal -offset left
> (set 't (table)) 
<TABLE#0> 
> (hash 'zero "zero") 
"zero"
.Ed
.Pp
.Pp
.Pp
.Ss keys: (keys expr1)
The "keys" intrinsic accepts one argument, which must evaluate to a table, and
returns a list of all the objects used as hash keys in in no particular order.
If the specified table is empty, the empty list is returned.
.Bd -literal -offset left
> (set 't (table)) 
<TABLE#0> 
> (hash t "0" "zero") 
"zero" 
> (hash t "1" "one")
"one" 
> (keys t) 
("1" "0")
.Ed
.Pp
.Pp
.Pp
.Ss values: (values expr1)
The "values" intrinsic accepts one argument, which must evaluate to a table,
and returns a list of all of the values stored in the table object in no
particular order.  If the specified table is empty, the empty list is
returned.
.Bd -literal -offset left
> (set 't (table)) 
<TABLE#0> 
> (hash t "0" "zero") 
"zero" 
> (hash t "1" "one")
"one" 
> (hash t "2" "two") 
"two" 
> (values t) 
("two" "zero" "one")
.Ed
.Pp
.Pp
.Pp
.Ss unhash: (unhash expr1 expr2)
The "unhash" intrinsic removes a key/value pair from a table.  It accepts
two arguments, the first of which must evaluate to the table to be
modified, while the second of which must evaluate to the key of the
key/value pair to be removed.  The "unhash" intrinsic always returns the
result of evaluating the second argument.
.Bd -literal -offset left
> (set 't (table)) 
<TABLE#0> 
> (hash t "0" "zero") 
"zero" 
> (unhash t "0") 
0 
> (keys t) 
()
.Ed
.Pp
.Pp
.Pp
.Ss lookup: (lookup expr1 expr2)
The "lookup" intrinsic is used to retrieve an object associated with an atom
in a table.  The function accepts two arguments.  The first argument must
evaluate to the table to be searched, while the second argument must evaluate
to an atom.  If another lisp object is associated with the second argument
in the specified table, the associated object is returned, otherwise the
empty list is returned.  Note that there is no way to tell the difference
between a key with no association and a key associated with the empty list.
.Bd -literal -offset left
> (set 't (table)) 
<TABLE#0> 
> (hash t "1" "one") 
"one" 
> (hash t "0" "zero")
"zero" 
> (lookup t "1") 
"one" 
> (lookup t "0")
"zero" 
> (lookup t "2")
()
.Ed
.Pp
.Pp
.Pp
.Ss sql_open: (sql_open expr)
The "sql_open" intrinsic opens a SQLite database file.  The function
accepts one argument which must evaluate to a string specifying the
filename for the database.  It will be created if it does not exist.
The opened database object is returned upon success.  Upon failure, a
string describing the error encountered is returned.  Database objects are
constants which evaluate to themselves.
.Pp
.Pp
.Pp
.Ss sql_close: (sql_close expr)
The "sql_close" intrinsic closes an open SQLite database file.  The
function accepts one argument which must evaluate to the opened database
object to close.  If the database is currently open the function closes
it and returns 1, otherwise, it returns 0.
.Pp
.Pp
.Pp
.Ss sql_exec: (sql_exec expr1 expr2)
The "sql_exec" function executes SQL commands on an opened SQLite database
file.  The function accepts two arguments, the first of which must evaluate
to the opened database object to query, and the second of which must
evaluate to a string specifying the SQL command to execute.  If the command
is successfully executed, a list is returned.  If the command would not
normally return any data, or the command returns the empty set, an empty
list is returned; otherwise, a list of lists is returned.  The first
sublist will contain the column keys, while each subsequent sublist will
contain one row of returned table data.  If an error is encountered during
execution of the SQL query, a string is returned describing the error.  
.Pp
Databases created with the SQLite library are largely typeless.  With one
exception, any type of data may be stored into any column type.  The typing
is considered a hint, and not a constraint.  Because of this all data
returned by this function are strings.  Numbers are returned as the string
representation of the appropriate value, and may be converted back to a
number with the "digitize" intrinsic.
.Pp
See http://www.sqlite.org for further details.
.Pp
.Pp
.Pp
.Ss cgi_accept: (cgi_accept)
The "cgi_accept" function must be used to by CGI scripts using FastCGI.
FastCGI allows CGI processes to persist between invocations, greatly
speeding up response times.  The toplevel of the CGI program calls
"cgi_accept" and is put to sleep until invoked, when "cgi_accept" will
return 1, indicating a successful invocation.  The function will return 0
when the socket to the webserver has been closed, therefore the toplevel of
the CGI program should be a loop which calls "cgi_accept" repeatedly until
it returns zero, when the progam should exit.  The function accepts no
arguments.  Note that all FastCGI I/O must be accomplished with one of the
following "cgi_" functions.  Additionally, most FastCGI programs will have
a significantly smaller memory footprint if garbage collection is
explicitly requested with the "gc" intrinsic after processing each client
request.  See the "client.fcgi" example program for an example of usage.
.Pp
.Pp
.Pp
.Ss cgi_finish: (cgi_finish)
The "cgi_finish" intrinsic can be used by scripts invoked by FastCGI, to
indicate the CGI program is finished sending data.  It is only necessary to
call "cgi_finish" if a CGI script needs to do some post-processing after
generating its output, and doesn't want to the client to wait until the
processing is done.  Normally, a script would simply call "cgi_accept"
again after it has finished generating its output, and be put to sleep
until invoked again.  The CGI script would call "cgi_accept" when finished
its post-processing activity.  The function does not accept any arguments, and
always returns 1.
.Pp
.Pp
.Pp
.Ss cgi_xstatus: (cgi_xstatus expr)
The "cgi_xstatus" intrinsic can be used by scripts invoked by FastCGI, to
set the apparent exit status of the CGI program.  When "cgi_accept" or
"cgi_finish" is subsequently called, the specified exit status will be
returned to the www server.  The function accepts one argument which must
evaluate to a number, which will be truncated into an 8-bit integer value
if it is outside the range of values from -128 to 127, and always returns
1.
.Pp
.Pp
.Pp
.Ss cgi_read: (cgi_read expr)
The "cgi_read" intrinsic must be used by CGI scripts invoked by FastCGI,
to read data that would normally be read from the standard input of a CGI
script invoked with the POST method.  The function accepts a single 
argument which must evaluate to a whole number specifying the number of
bytes to read.  The function returns the characters read as a string.
If EOF is encountered, the function returns 0.
.Pp
.Pp
.Pp
.Ss cgi_print: (cgi_print expr ...)
The "cgi_print" intrinsic must be used by CGI scripts invoked by FastCGI,
to send output data to the client.  It interprets it arguments exactly as
the "print" intrinsic does.  The function always returns 1.
.Pp
.Pp
.Pp
.Ss cgi_newline: (cgi_newline)
The "cgi_newline" intrinsic must be used by CGI scripts invoked by FastCGI,
to send line terminators to the client.  A carriage return character
followed by a line feed (newline) character are sent to the client.  The
function accepts no arguments, and always returns 1.
.Pp
.Pp
.Pp
.Ss cgi_getenv: (cgi_getenv expr)
The "cgi_getenv" intrinsic must be used by CGI scripts invoked by FastCGI,
to access environment settings.  The function accepts one argument which
must evaluate to a string naming a variable to be looked up in the FastCGI
environment, and returns a string representing the value of that variable,
if it is set, or 0 if is not set.
.Pp
.Pp
.Pp
.Ss cgi_warn: (cgi_warn expr ...)
The "cgi_warn" intrinsic must be used by CGI scripts invoked by FastCGI, to
send error messages to the server's log file.  The function accepts any
number of arguments, are of wich are all evaluated.  The result of each
evaluation is written to the stderr stream of the FastCGI connection.
After all the arguments have been processed, a newline character is written
to the stream.  The output will appear to the WWW server to have come from
the CGI program's standard error.  The function always returns 1.
.Pp
.Pp
.Pp
.Ss cgi_sendfile: (cgi_sendfile expr)
The "cgi_sendfile" intrinsic accepts one argument which must evaluate to a
string specifying a filename, whose content the programmer desires to send to
the client over the active fcgi connection.  This is useful in the situtation
where one wishes to send a client data from a file which resides outside of
the server's root directory, and so cannot be specified in a "src" or "href"
attribute in the page being created.  The function returns 1 upon success.
Upon failure of the "open" system call, a string is returned describing the
error.  Other errors stop evaluation.
.Pp
.Pp
.Pp
.Ss stack: (stack [expr])
The "stack" intrinsic creates a new stack object.  The function accepts one
optional argument, which, if present, must evaluate to a positive integer
specifying a number of elements to preallocate on the stack.  Each element
will be set to the empty list.  Omitting the optional argument is the same
as invoking (stack 0).  The newly-created stack object is returned.
.Bd -literal -offset left
> (set 's (stack))
<STACK1>
> (used s)
0
> (set 's (stack 10))
<STACK2>
> (used s)
10
.Ed
.Pp
.Pp
.Pp
.Ss push: (push expr1 expr2)
The "push" intrinsic pushes an object onto the top of a stack.  The
function accepts two arguments, the first of which must evaluate to the
stack to be affected, while the second argument may be any lisp object.
The function returns the result of evaluating the second argument.
.Bd -literal -offset left
> (set 's (stack))
<STACK1>
> (push s 'foo)
foo
> (index s 0)
foo
.Ed
.Pp
.Pp
.Pp
.Ss pop: (pop expr)
The "pop" intrinsic removes an object from the top of a stack.  The
function accepts one argument, which must evaluate to the stack to be
affected.  The removed object is returned.  When the stack is empty, the
empty list is returned.  Note the only way to tell the difference between
an empty stack and one which has the empty list stored in its top element,
is to invoke the "used" intrinsic.
.Bd -literal -offset left
> (set 's (stack))
<STACK1>
> (push s 'foo)
foo
> (push s 'bar)
bar
> (pop s)
bar
> (pop s)
foo
> (pop s)
()
> (used s)
0
.Ed
.Pp
.Pp
.Pp
.Ss unshift: (unshift expr1 expr2)
The "unshift" intrinsic adds a new element onto the bottom of a stack.  The
function accepts two arguments, the first of which must evaluate to a stack,
while the second may evaluate to any lisp object.  The result of evaluating
the second argument is prepended to the stack, and also returned by the
function.
.Bd -literal -offset left
> (set 's (assign (stack) '(1 2 3 4)))
<STACK1>
> (flatten s)
(1 2 3 4)
> (unshift s 0)
0
> (flatten s)
(0 1 2 3 4)
.Ed
.Pp
.Pp
.Pp
.Ss shift: (shift expr)
The "shift" library function removes an element from the bottom of a stack.
The function accepts on argument which must evaluate to stack.  If the stack
is empty, the empty list is returned, otherwise the removed element is
returned.
.Bd -literal -offset left
> (set 's (assign (stack) '(0 1 2 3 4)))
<STACK1>
> (flatten s)
(0 1 2 3 4)
> (shift s)
0
> (shift s)
1
> (flatten s)
(2 3 4)
.Ed
.Pp
.Pp
.Pp
.Ss index: (index expr1 expr2)]
The "index" intrinsic fetches the object stored at a specified index in a
stack.  The function accepts two arguments, the first of which must
evaluate to the stack to be accessed, while the second must evaluate to a
whole number specifying the desired element on the stack.  The bottom
location on a stack has an index value of zero, and the index value of the
object on the top of the stack is one less than the number of elements on
the stack.  Specifying an index value of less than zero, or more than the
index of the last element on the stack, generates an error which stops
evaluation.
.Bd -literal -offset left
> (set 's (assign (stack) '(a b c)))
<STACK1>
> (index s 0)
a
> (index s 2)
c
> (index s 3)
<INDEX>: index 3 out of range.
> 
.Ed
.Pp
.Pp
.Pp
.Ss topidx: (topidx expr)
The "topidx" intrinsic accepts one argument which must evaluate to a
stack, and returns the index value of the top element on the stack, or -1
if the stack is empty.
.Bd -literal -offset left
> (topidx (stack 1))
0
.Ed
.Pp
.Pp
.Pp
.Ss used: (used expr)
The "used" intrinsic obtains the number of elements currently on a
specified stack.  The function accepts one argument which must evaluate to
the stack to be queried, and returns a whole number describing the number
of elements currently on that stack.
.Bd -literal -offset left
> (used (stack 10))
10
.Ed
.Pp
.Pp
.Pp
.Ss store: (store expr1 expr2 expr3)
The "store" intrinsic stores an object into a specified element of a stack.
The function accepts three arguments, the first of which must evaluate to
the stack to be affected, while the second argument must evaluate to a
whole number specifying the element of the stack to overwrite, and the
third argument may evaluate to any lisp object.  The result of evaluating
the third argument is stored in the specified index of the specified stack,
and the result of evaluating the third argument is returned.  Specifying an
index value of less than zero, or more than the index of the last element
on the stack, generates an error which stops evaluation.
.Bd -literal -offset left
> (set 's (stack 3))
<STACK1>
> (store s 0 'foo)
foo
> (store s 1 'bar)
bar
> (store s 2 'wumpus)
wumpus
> (flatten s)
(foo bar wumpus)
> (store s 3 'error)
<STORE>: index 3 out of range.
>
.Ed
.Pp
.Pp
.Pp
.Ss clear: (clear expr1 expr2)
The "clear" intrinsic allows the user to remove and discard multiple elements
from the top of a specified stack.  The function accepts two arguments, the
first of which must evaluate to a stack object, while the second must evaluate
to a whole number.  The second argument specifies the number of arguments to
remove from the top of the stack.  "clear" discards the removed elements and
always returns 1.
.Bd -literal -offset left
> (setq s (assign (stack) '(1 2 3 4 5)))
<STACK#1>
> (clear s 4)
1
> (flatten s)
(1)
> (assign s '(1 2 3 4 5))
<STACK#1>
> (clear s (used s))
1
> (flatten s)
()
.Ed
.Pp
.Pp
.Pp
.Ss assign: (assign expr1 expr2)
The "assign" library function allows the user to store all the elements of
a list into a stack at once, starting at index zero.  The previous contents
of each affected element are overwritten.  If the number of elements on the
stack is too small to hold all the objects in the list, the function pushes
new elements onto the stack until it can hold the entire list, before
performing the assignments.  If the number of elements on the stack is
greater than the number of items in the list, those elements on the stack
starting at the index value equal to the length of the list, and continuing
to the top of stack, inclusive, remain unaffected.  The function returns
the stack object affected.
.Bd -literal -offset left
> (set 's (assign (stack) '(a b c d e)))
<STACK1>
> (used s)
5
.Ed
.Pp
.Pp
.Pp
.Ss flatten: (flatten expr)
The "flatten" library function returns the elements of a stack in ascending
order, as a list.  The function accepts one argument which must evaluate to
the stack to be queried.
.Bd -literal -offset left
> (flatten (assign (stack) '(a b c d e)))
(a b c d e)
.Ed
.Pp
.Pp
.Pp
.Ss child_open: (child_open expr)
The "child_open" intrinsic forks another process which may be communicated
with by the "child_read" and "child_write" intrinsics.  Only one inferior
process may be running at any one time.  The function accepts one argument
which must evaluate to a string specifying a command line to pass to the 
shell (/bin/sh) to run, and returns 1.  Any errors encountered while trying
to fork a new process will stop evaluation.  If the child process cannot
find the specified program, or is not able to run it, it will exit.  The
user should therefore check for a successful launch of the specified
program by invoking "child_running" after invoking "child_open".
.Pp
.Pp
.Pp
.Ss child_running: (child_running)
The "child_running" intrinsic accepts no arguments, and returns 1 if an
inferior process is running; otherwise, the function returns 0.
.Pp
.Pp
.Pp
.Ss child_ready: (child_ready)
The "child_ready" intrinsic accepts no arguments, and returns 1 if data is
waiting to be read from an inferior process; otherwise the function returns
0.  If an inferior process has not been started with the "child_open"
intrinsic, an error will be generated, stopping evaluation.
.Pp
.Pp
.Pp
.Ss child_wait (child_wait)
The "child_wait" intrinsic accepts no arguments and blocks until data is
ready from a child process for "child_read" to consume.  If no child
process is running, an error is generated which will stop evaluation.
.Pp
.Pp
.Pp
.Ss child_close: (child_close)
The "child_close" intrinsic terminates an inferior process launched by
"child_open".  The function accepts no arguments, and always returns 1.  If
there is no inferior process running, the function does nothing.
.Pp
.Pp
.Pp
.Ss child_eof: (child_eof)
The "child_eof" intrinsic closes the writable half of the connection to
child process opened with "child_open".  The function accepts no arguments
and returns 1 upon success.  The child process will read EOF on its next
read from its standard input.  This can be useful when working with
programs which buffer data.  Any subsequent attempt to write to the child
with "child_write" will generate an error which will stop evaluation.  Note
that a connection which has had its writable half closed with "child_eof"
still needs to be fully-closed by invoking "child_close" on it, when one is
finished with the child.
.Pp
For an example of the use of this intrinsic, consider a situation where one
is sending lines of text to the fmt utility.  This program will buffer
input text until it has enough to print a full line of output, unless it
encounters a blank line or EOF, when any buffered text will be output to
form a short line.  In the situation where one has sent data to the
utility, and it has formatted all the data except for the tail end which is
not long enough to form a complete line, the program will block forever
until it reads more data or EOF from stdin.  By invoking "child_eof" we can
cause fmt to read EOF on stdin and print the last of its buffered output.
If we invoked "child_close" we would close both halves of the full-duplex
connection, and so be unable to read that last short line of data back from
the child.
.Pp
.Pp
.Pp
.Ss child_write: (child_write expr1 ...)
The "child_write" intrinsic writes a list of strings to an inferior process
launched by "child_open".  The function accepts one or more arguments, which
must all evaluate to strings, and writes them to the inferior process.  The
"child_write" function returns 1 on success.  Any errors encountered will
stop evaluation.
.Pp
.Bd -literal -offset left
> (child_open "/usr/local/bin/munger")
1
> (for (a 1 10)
>> (child_write a (char 10)))
1 ; this is the return value of the "child_write"
>> (print (child_read)))
1
2
3
4
5
6
7
8
9
10
.Ed
.Pp
.Pp
.Pp
.Ss child_read: (child_read)
The "child_read" intrinsic reads up to 128 characters of data emmitted by an
inferior process, and returns it as a string.  The function accepts no
arguments.  If "child_read" is invoked when no inferior process is running,
an error will be generated which will stop evaluation.  If no data can be
read after 30 seconds, "child_read" will return the empty string.  Zero is
returned if EOF is read from the inferior process.
.Bd -literal -offset left
> (child_open "munger")
1
> (child_write "(set 'foo 'bar)" (char 10))
1
> (child_read)
"bar
"
1
.Ed
.Ss clearscreen: (clearscreen)
The "clearscreen" intrinsic accepts no arguments, and if stdout is
connected to a terminal device, clears the screen.  1 is returned on
success.  Any error encountered will stop evaluation.
.Pp
.Pp
.Pp
.Ss clearline: (clearline expr1 expr2)
The "clearline" intrinsic accepts two arguments, both of which must
evaluate to whole numbers.  If stdout is connected to a cursor-addressable
terminal device, it will clear the line specified by the first argument,
starting at the column specified by the second argument, to the end of
the line.  The coordinates for screen lines and columns start at 0.  The
postion 0,0 is the top leftmost position on the screen.  The maximum values
for the terminal device can be ascertained with the "lines" and "cols"
intrinsics.  1 is returned on success.  Any error encountered will stop the
interpreter.
.Pp
.Pp
.Pp
.Ss goto: (goto expr1 expr2)
The "goto" intrinsic accepts two arguments, both of which must evaluate to
whole numbers.  If stdout is connected to a cursor-addressable terminal device,
the function places the cursor at the specified screen coordinates.  The
coordinates for screen lines and columns start at 0.  The postion 0,0 is
the top leftmost position on the screen.  The maximum values for the
terminal device can be ascertained with the "lines" and "cols"
intrinsics.  1 is returned on success.  Any error encountered will stop the
interpreter.
.Pp
.Pp
.Pp
.Ss getchar: (getchar)
The "getchar" intrinsic accepts no arguments.  It does a blocking read on
stdin until a character can be read, when it returns a whole number in the
range of 0-255, representing the character code of the input character.
If stdin is a terminal device, it will be taken out of canonical mode, so
that unbuffered, uninterpreted data may be read.  If EOF is encountered, then
"getchar" returns -1.  If a SIGWINCH is received by the interpreter while
waiting for data, the function returns -2 immediately.  Any error encountered
will stop evaluation.  "getchar" is intended for use in creating interfaces 
which use character-I/O.  If you just want to read a character from stdin, and
stdin is redirected onto a file, use (getchars 1) instead.  If you are writing
a CGI script, you must use (getchars 1) because only "getchars" can handle the
situation where the read() system call returns a short count when reading from
a socket.
.Pp
.Pp
.Pp
.Ss pushback: (pushback expr1)
The "pushback" intrinsic accepts one argument which must evaluate to a
whole number in the range 0-255.  It causes the next subsequent invocation
of "getchar" to return the number pushed back.  After returning this value,
subsequent invocations of "getchar" will read from the terminal again.
1 is returned on success.  Any error encountered stop evaluation.  Note that
"pushback" does not work with "getchars", only "getchar".
.Pp
.Pp
.Pp
.Ss display: (display expr1 expr2 expr3)
The "display" intrinsic aids the implementation of interactive buffer
inspection tools.  The function accepts three arguments, all of which must
evaluate to whole numbers.  If stdout is connected to a cursor-addressable
terminal device, the function will print buffer lines, one per screenline,
starting with the buffer line whose index value corresponds to the first
argument, and continuing with subsequent buffer lines, until it has printed
one less many buffer lines as there are screen lines, or until it runs out
of buffer lines, if there are not enough.  If it runs out of lines, the
function will print single tilde characters on each of the remaining
screenlines, except for the last.  If the first argument is zero, "display"
prints a full screen of tildes, and returns.
.Pp
The second argument specifies a buffer column to start printing with.  If
non-zero, only the portions of the lines from the specified column onward
will be printed.  Note the second argument does not specify a screen
column, but a buffer column, and that tab expansion is performed before the
slice is taken, according to the value of the third argument, discussed
below.  Lines will always be printed starting at screen column 0.  Lines
longer than the terminal width are truncated to the terminal width.
.Pp
The third argument specifies the tabstop periodicity.  A value of 4, for
example, indicates that tabs occur every 4 characters in a line.  Any tabs
found in the specified lines will be expanded according to this value
before truncation and printing.  For the details of tab expansion, see the
description of the "expand" intrinsic, elsewhere in this document.
.Pp
1 is returned on success.  Any errors encountered stop evaluation.
.Pp
.Pp
.Pp
.Pp
.Ss boldface: (boldface)
The "boldface" intrinsic turns on boldface mode of the terminal connected
to stdout.  The function accepts no arguments, and always returns 1.
.Pp
.Pp
.Pp
.Ss normal: (normal)
The "normal" intrinsic turns off boldface mode of the terminal connected
to stdout.  The function accepts no arguments and always returns 1.
.Pp
.Pp
.Pp
.Ss hide: (hide)
The "hide" intrinsic accepts no arguments, and if stdout is connected to a
terminal device capable of having its cursor made invisible, the function
hides the cursor.
.Pp
.Pp
.Pp
.Ss show: (show)
The "show" intrinsic accepts no arguments, and if stdout is connected to a
terminal device capable of having its cursor made invisible, the function
shows the cursor.
.Pp
.Pp
.Pp
.Ss pause: (pause expr1)
The "pause" intrinsic accepts one argument which must evaluate to a
whole number no larger than 999999, specifying a number of microseconds for
the interpreter to sleep.  The interpreter returns when data is waiting on
stdin or when the time value has expired.
.Pp
.Pp
.Pp
.Ss scrollup: (scrollup)
If the device connected to stdin is cursor-addressable, the "scrollup"
intrinsic scrolls the screen lines upward by one line.  The line at the top
of the screen is lost, while the line at the bottom of the screen becomes
blank.
.Pp
.Pp
.Pp
.Ss scrolldn: (scrolldn)
If the device connected to stdin is cursor-addressable, the "scrolldn"
intrinsic scrolls the screen lines downward by one line.  The last line is 
lost, while the first line becomes blank.
.Pp
.Pp
.Pp
.Ss insertln: (insertln)
If the device connected to stdin in cursor-addressable, the "insertln"
intrinsic scrolls the lines on the screen from the line the cursor is on,
to the last line on the screen, inclusive, downward by one line. The line
the cursor is on is cleared, while the last line on the screen is lost.
.Pp
.Pp
.Pp
.Ss printer: (printer)
The "printer" intrinsic turns on the lisp printer, if it has been turned off
with an invocation of the "noprinter" intrinsic.  The result of evaluating an
expression at toplevel is discarded unless the printer is turned on.  The
printer is turned on by default.
.Pp
.Pp
.Pp
.Ss noprinter: (noprinter)
The "noprinter" intrinsic turns off the lisp printer, if it is turned on.  The
result of evaluating an expression at toplevel is discarded unless the printer
is turned on.  The printer is turned on by default.
.Pp
.Pp
.Pp
.Ss exec: (exec expr)
The "exec" intrinsic overlays a new process overtop of the interpreter
process, similarly to how the command of the same name works in the shell.
The function accepts one argument which must evaluate to a string, and
attempts to use the execv() system call to run the shell (/bin/sh -c) with
the command-line specified in the argument string.  Upon success, the Munger
interpreter process is abandoned, and the shell process starts running with
the same process id.  It "replaces" the interpreter.  This is useful when a
script is finished its work and wishes to run another program to do some
further processing.
.Pp
.Pp
.Pp
.Ss truncate: (truncate expr)
The "truncate" intrinsic alters the length of a file connected to the standard
output of the interpreter.  The function accepts one argument which must
evaluate to a an integer specifying the new length of the file.  If the
specified length is greater than the length of the file, the file will be
extended and the extended portion filled with zeros.  The function returns 1
on success, or if an error is encountered, a string describing the error.
.Bd -literal -offset left
> (with_output_file_appending "HOW_TO_EXTEND_IT"
>> (when (writelock)
>>> (truncate 100)))
1
> (with_input_file "HOW_TO_EXTEND_IT"
>> (setq line (getchars 1000)))
"ADDING INTRINSICS TO THE INTERPRETER
------------------------------------

Under the hood, Munger is"
> (length line)
100
.Ed
.Pp
.Pp
.Pp
.Ss dynamic_let: (dynamic_let (symbol expr) expr1 [expr2...])
Intrinsic "dynamic_let" allows the creation of bindings with dynamic scope,
which is to say, such a binding is globally visible for the time the
dynamic_let is executing.  A dynamically-scoped variable in Munger is a global
which will cease to exist when the "dynamic-let" exits, or if a global of the
same name existed previously, revert to its former value.  The bindings
created by "dynamic_let" cannot be captured by closures.  A lexical binding of
the same name as a dynamic binding will shadow the dynamic binding.  This
means a binding in an exterior "let" form can shadow the binding in an
interior "dynamic_let".  The form accepts two or more arguments, the first of
which is not evaluated and must be a two-element list consisting of a symbol
followed by any expression.  The expression will be evaluated in the current
scope and the result bound to the symbol for the duration of the
"dynamic_let".  The remaining arguments are then evaluated, in order, with the
new binding in effect.  When the form terminates the dynamic binding is
removed.  The value of the last expression evaluated is returned by
"dynamic_let".
.Pp
The purpose of "dynamic_let" is to allow one to make temporary changes to
global variables without having to save the previous value and restore it
afterwards.  By definition, a global is a variable one wishes to be globally
visible.  One would only knowingly mask it with a local binding if one wished
to have some local code find a different value bound to the same symbol.  A
"let" is sufficient for this purpose, but if one wishes to have other
functions which the local code may invoke to also see the changed value, then
"dyamic_let" must be used.  See the "google.munger" example program for an
example of usage.  There, a global named "document" is bound to a
recursively-structured alist representing an XML document.  "dynamic_let" is
used to temporarily bind "document" to a sublist of the alist, to reduce the
amount of tree traversal the funtions in xmlquery have to perform.
.Bd -literal -offset left
> (defun f () a)
<CLOSURE#57>

> (dynamic_let (a 10)
>> (f))
10

> (f)
evaluate: symbol a not bound.
f: error evaluating body expression 1.

> (defun a (x) 'foobar)
<CLOSURE#58>

> (defun g (x) (a x))
<CLOSURE#59>

> (dynamic_let (a (lambda (x) x))
>> (g 23))
23

> (g 23)
foobar

> (dynamic_let (a 10)
>> (defun h () a))
<CLOSURE#60>
> (h)
evaluate: symbol a not bound.
h: error evaluating body expression 1.

; Exterior "let" is shadowing interior "dynamic_let":

> (let ((a 12))
>> (dynamic_let (a 10)
>>> (print a)
>>> (newline)))
12
1 ; return value of "newline"
.Ed
.Pp
.Pp
.Pp
.Ss basename: (basename expr)
The "basename" intrinsic returns just the filename portion of a specified path.
The function accepts one argument which must evaluate to a string and returns
a string.  If the argument string does not have a filename component,
"basename" returns the empty string.  NOTE, the path specified by the argument
string does not have to exist in the filesystem.  This is a string manipulation
function only.
.Pp
.Pp
.Pp
.Ss dirname: (basename expr)
The "dirname" intrinsic returns just the directory portion of a specified path.
The function accepts one argument which must evaluate to a string and returns
a string.  If the argument does not have a directory component "." is returned.
NOTE, the path specified by the argument string does not have to exists in 
the filesystem.  This is a string manipulation function only.
.Pp
.Pp
.Pp
.Ss chmod: (chmod expr1 expr2)
The "chmod" intrinsic may be used to change the permissions associated with a
specified file.  The function accepts two arguments, both of which must
evaluate to strings.  The first argument must be a new mode specification of
the form accepted by chmod(1), in either symbolic or octal form.  The second
argument must evaluate to the filename of the file to be affected.  "chmod"
returns 1 on success; otherwise it returns a string describing an error
condition.
.Pp
.Pp
.Pp
.Ss chown: (chown expr1 expr2 expr3)
The "chown" intrinsic may be used to change the owner (if the euid of the
interpreter is the superuser's), and/or the group associated with a specified
file.  The function accepts three arguments, which all must evaluate to
strings.  The first argument must be the name of the new owner, or the empty
string if the owner is not to be changed.  The second argument must be name of
the new group, or the empty string if the group is not to be changed.  The
third argument must be the filename of the file to be affected.  "chown"
returns 1 upon success; otherwise, it returns a string describing an error
condition.  Only the superuser may change the ownership of files.
.Pp
.Pp
.Pp
.Ss crypt: (crypt expr)
The "crypt" intrinsic is a frontend to the crypt(3) library function.  It 
accepts one argument which must evaluate to a string and encrypts it using
the default scheme used to encrypt passwords in the user databas.  The
encrypted string is returned.  Any error encountered by crypt(3) will stop
evaluation.
.Pp
.Pp
.Pp
.Ss checkpass: (checkpass expr1 expr2)
The "checkpass" intrinsic verifies a user and password pair are correct for the
system on which it is running.  The function accepts two arguments which must
both evaluate to strings, the first of which specifies the user name, and the
second of which specifies the password of that user.  The function will always
return 0 unless the euid of the interpreter is 0 (the superuser), when it will
return 1 if the user name and password are correct, or 0, if they are not.
.Pp
.Pp
.Pp
.Ss setuid: (setuid expr)
The "setuid" intrinsic is used to change the uid and euid of the interpreter
process.  It accepts one argument which must evaluate to a string specifying
the user to change to.  The function always returns 0 unless the euid of the
interpreter is 0 (the superuser), when it will change to the specified user 
and return 1, if the user exists, or 0 otherwise.  It is not possible to
switch back to the superuser after invoking "setuid" to become a
non-priveleged user.  Further invocations of "setuid" will return 0.
.Pp
.Pp
.Pp
.Ss seteuid: (seteuid expr)
The "seteuid" intrinsic is used to change the euid of the interpreter process
if it is running set-user-id.  It accepts one argument which must evaluate to a
string specifying the user to change to, and returns 1 upon success or 0 upon
failure.  The euid may be switched between the real uid and the set-user-id,
of a set-user-id interpreter.  If the interpreter is not running setuid, then
the uid, the euid, and the saved set-user-id will all be the same and
therefore it will not be possible to change the euid.
.Pp
.Pp
.Pp
.Ss getuid: (getuid)
The "getuid" intrinsic accepts no arguments and returns a two element list
consisting of the name of the real user the interpreter is running as,
followed by the numerical uid of that user.  If an unforseen error occurs
which prevents the interpreter from determining its own uid, the function
returns the empty list.
.Pp
.Pp
.Pp
.Ss geteuid: (geteuid)
The "geteuid" intrinsic accepts no arguments and returns a two element list
consisting of the name of the effective user the interpreter is running as,
followed by the numerical uid of that user.  If an unforseen error occurs
which prevents the interpreter from determining its own euid, the function
returns the empty list.
.Pp
.Pp
.Pp
.Ss getgid: (getgid)
The "getgid" intrinsic accepts no arguments, and returns a two element list
consisting of the name of the primary group of the user the interpreter is
running as, followed by the numerical gid of that group.  If an unforseen
error occurs which prevents the interpreter from determining its gid, then
the function returns the empty list.
.Pp
.Pp
.Pp
.Ss seek: (seek expr1 expr2 expr3)
The "seek" intrinsic is used to move the file pointer of a file connected to
the standard input or standard output.  The function accepts three arguments,
the first of which must evaluate to either 0 or 1, and specifies whether the
seek operation should affect the file pointer of stdin or stdout,
respectively.  The second argument must evaluate to a integer specifying the
number of characters by which the file pointer should be adjusted, and the
third argument must evaluate to one of the following three strings:
"SEEK_SET", "SEEK_CUR", or "SEEK_END", and specifies the position which the
adjustment is relative to.  "SEEK_SET" indicates the seek operation should
seek from the beginning of the file.  "SEEK_CUR" indiciates the seek operation
should seek from the current position of the file pointer.  "SEEK_END"
indicates the seek operation should seek from the end of the file.  Upon
success the function returns the number of characters from the beginning of
the file corresponding to the new location of the file pointer.  Further reads
or writes to the stream will happen relative to the new position of the file
pointer.  Any errors encountered will stop evaluation.  Seeking past the end
of a file connected to stdout will cause the file to be automatically
extended, with the new portion filled with zeroes.
.Pp
NOTE: calling "getline" on stdin will cause subsequent invocations of "seek"
to return unexpected values, as "getline" does its own buffering.  If the
user wishes to intersperse the reading of data from stdin and calls to "seek"
on stdin, the "getchars" intrinsic must be used to perform the read
operations.
.Pp
.Pp
.Pp
.Ss getchars: (getchars expr)
The "getchars" intrinsic is used to read a specific number of characters
from the stream connected to stdin.  It accepts one argument which must
evaluate to a whole number specifying the number of characters to read.
Upon success, the characters read are returned as a string.  Any errors
encountered will stop evaluation.  If "getchars" encounters EOF while reading,
less than the desired amount of characters may be returned.  The next invocation
of the function will return the number 0, indicating it has reached EOF.
.Pp
NOTE: Mixing calls to "getline" and "getchars" will result in unexpected results,
as "getline" performs its own buffering.  When reading from an single input
source, use one or the other exclusively.  Both "getchars" and "getline" will
handle the situation where the read() system call returns a short count when
reading from a socket, by continually invoking read() until the desired amount
of data has been read, or EOF has been reached.
.Pp
.Pp
.Pp
.Ss readlock: (readlock)
The "readlock" intrinsic is used to obtain a shared lock on a file connected
to stdin.  The function accepts no arguments and returns 1 upon success, or 0
if the file is already locked by another process. If the lock cannot be
obtained for any other reason, the function returns a string describing the
error.
.Pp
.Pp
.Pp
.Ss writelock: (writelock)
The "writelock" intrinsic is used to obtain an exclusive lock on a file
connected to stdout.  The function accepts no arguments and returns 1 upon
success, or 0 if the file is already locked by another process.  If the lock
cannot be obtained for any other reason, the function returns a string
describing the error.
.Pp
.Pp
.Pp
.Ss unlock: (unlock expr)
The "unlock" intrinsic is used to release a lock obtained by the "readlock"
or "writelock" intrinsics.  The function accepts one argument which must
evaluate to either 0 or 1, specifying whether a lock on stdin or stdout is to
be released, respectively.
.Pp
.Pp
.Pp
.Ss hostname: (hostname)
The "hostname" intrinsic accepts no arguments and returns the name of host
it is running on, or a string describing an error condition it encountered
while attempting to retrieve the hostname.
.Pp
.Pp
.Pp
.Ss symlink: (symlink expr1 expr2)
The "symlink" intrinsic creates a symbolic link to a pre-existing filesystem entity.
The function accepts two arguments, both of which must evaluate to strings.  The first
argument specifies the pre-existing filesystem entity, while the second argument names
the symbolic link.  "symlink" returns 1 upon success, or a string describing an error
condition, if the symlink() system call failed.
.Pp
.Pp
.Pp
.Ss gecos: (gecos expr)
The "gecos" intrinsic queries the user database for the value of the gecos field
for a specified account.  The function accepts one argument which must be a
string specyfing the user name associated with the account.  If such a user
exists, the value of the gecos field is returned as a string, otherwise the
empty string is returned.  The gecos field is used to store personal
information about the user.  Traditionally, it held 4 comma-separated fields
containing the user's full name, office location, work phone number, and home
phone number, but the system does not care what goes into the gecos field.  Most
admininstrators nowadays simply place the user's full name there.
.Pp
.Pp
.Pp
.Ss record: (record expr)
The "record" intrinsic creates a fixed-size unidimensional array.  Records
are a more space-efficient means of representing fixed-size aggregate
types.  The one argument must evaluate to a positive integer specifying the
size of the array.  The items of the record are all preset to the empty
list upon creation.
.Pp
.Pp
.Pp
.Ss getfield: (getfield expr1 expr2)
The "getfield" intrinsic is used to retrieve an item from a record.  It
accepts two arguments, the first of which must evaluate to a record, while
the second must evaluate to a positive integer specifying the index of the
desired item in the array.  The object at that index is returned.
.Pp
.Pp
.Pp
.Ss setfield: (setfield expr1 expr2 expr3)
The "setfield" intrinsic is used to insert an object into a record.  The
function accepts three arguments.  The first argument must evaluate to the
record to be affected.  The second argument must evaluate to a positive
integer specifying the index location to overwrite.  The third argument can
evaluate to any lisp object and will be inserted into the first argument at
the location specified by the second argument.  The evaluated third argument
is returned.
.Pp
.Pp
.Pp
.Ss extend: (extend expr1 expr2)
The "extend" intrinsic adds a new binding to the currently-active local
environment.  The function accepts two arguments, the first of which must
evaluate to a symbol, while the second of which may evaluate to any lisp
object.  The symbol is added to the local environment and bound to the
result of evaluating the second argument, replacing any pre-existing local
binding to that symbol.  Note that if the pre-existing symbol occurs free
in the current lexical environment, then it is shadowed, not replaced.  The
result of evaluating the second argument is returned.  The intent beind the
inclusion of this intrinsic in the interpreter is to allow extensions to
the current lexical environment without the use of "let" and friends, for
increased efficiency.
.Bd -literal -offset left
> (defun fact (n)
>> (extend 'a 1)
>> (while (> n 1)
>>>   (setq a (* n a))
>>>   (dec n))
>> a)
<CLOSURE#23>
> (fact 10)
3628800
.Ed
.Pp
Any lambda-expressions closing over the current lexical environment will
"see" the new binding, because closures do not simply close over the
lexical bindings visible at the time of their creation, but rather the
lexical environments visible at the time of their creation.  The difference
between these two notions, is that environments may be dynamically extended
with "extend" to contain a superset of the bindings visible at creation.
.Pp
If the closure is applied before an invocation of "extend" it will not see
the binding created, since it will be created in the future, but if that
VERY SAME CLOSURE is applied after an invocation of "extend" it will see
the new binding created.  The new binding suddenly appears in the current
lexical environment.
.Pp
This means a closure bound to a new local via "extend" will "see" its own
binding when it is applied, and therefore may call itself by name.  The
extent of bindings created by "extend" is unlimited, but may be limited by
wrapping the expressions in which they occur with an invocation of
"dynamic_extent" described elsewhere in this document.
.Pp
.Pp
.Pp
.Ss gc: (gc)
The "gc" intrinsic sets the garbage collector to run on the next
evaluation.  The garbage collector normally runs once every 65536
evaluations.  Judicious invocations of this function may allow the user to
decrease the memory consumption of his or her program.
.Pp
.Pp
.Pp
.Ss dynamic_extent: (dynamic extent ...)
The "dynamic_extent" intrinsic limits the extent of additions to the
current lexical environment made with the "extend" intrinsic.  The
intrinsic accepts zero or more arguments.  If no arguments are supplied,
"dynamic_extent" does nothing and returns 1.  If arguments are supplied,
they are evaluated in order, and the result of evaluating the last
expression is returned.  If no lexical environment exists at the time of
invocation, an error will be generated which will stop evaluation.  When
"dynamic_extent" finishes, any additions made to the current lexical
environment by the expressions in its body are removed.  Combinations of
"dynamic_extent" and "extend" can replace occurrences of "let", "letn", or
"labels" inside functions where the bodies of those forms do not contain
closures, or which contain closures which do not need to close over the
new bindings, which will not persist beyond the extent of "dynamic_extent".
.Pp
In this example the new bindings introduced for "b" and "a" do not have
their extent limited, and any closures formed in the body of the "let"
would continue to "see" these bindings when the "let" returned, even if the
closures were closed before the invocations of "extend".  The binding to
"c" however, has its extent limited to the extent of the invocation of
"dynamic_extent".  If no new bindings are introduced via "extend" inside an
instance of "dynamic_extent", then it effectively does nothing.
.Bd -literal -offset left
> (let ((a 10))
>> (extend 'b (* a a))

>> (dynamic_extent
>>> (extend 'c (* b b))
>>> (print c)
>>> (newline))

>> (print (bound 'c))
>> (newline))
10000 ; first "print"
0     ; second "print"
.Ed
.Pp
.Pp
.Pp
.Ss gc_freq: (gc_freq expr)
The "gc_freq" intrinsic allows the programmer to change the rate at which
garbage collection occurs.  In the interpreter, GC happens after a specific
number of calls to evaluate().  The value of this period is stored in the C
global gc_frequency.  "gc_freq" accepts one argument, which must evaluate
to a positive fixnum specifying a new value for gc_frequency.  Increasing
gc_frequency will cause the interpreter to run faster up to a point, but
consume more memory, while decreasing the value of gc_frequency will cause
the interpreter to run more slowly but consume less memory.  The function
returns the old value of gc_frequency.  Setting gc_frequency to zero will
disable garbage collection.  This is useful to me when debugging the
interpreter.
